#!/bin/sh
# Script to run the EXAMPLE_FWD code
#
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# This script runs only ONE test for NOAA-16 AVHRR
# 
# R Saunders Jan 2008
# Updated for RTTOV-10 J Hocking June 2010

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######
# Relative to the rttov_test directory:
TEST=./test_example_fwd.1

# Relative to the rttov_test/${TEST} directory:
BINDIR=../../$BIN
DATART=../../rtcoef_rttov10/rttov7pred51L
REF_TEST=../test_example_fwd.2
###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


CWD=`pwd`
cd $TEST

# Test case
PLATFORM=noaa
PLATFORM_N=1
INST=avhrr
INST_N=5
SATID=16
FILE_PROFILE=./prof.dat
ZENANGLE=0.
SURF_TYPE=1     # Sea
WATER_TYPE=1    # Ocean
NLEVELS=51      # Num of profile levels

echo " "
echo " "
echo " Test forward  ${INST} "
echo " "

echo  "Platform   ${PLATFORM}"
echo  "Instrument ${INST}"
echo  "Satellite  ${SATID}"
echo  "File for profile  ${FILE_PROFILE}"
echo  "Zenith angle ${ZENANGLE}"
echo  "Azimuth angle ${AZANGLE}"
echo  "Surface type ${SURF_TYPE}"
echo  "Water type ${WATER_TYPE}"
echo  "Number of levels of profile ${NLEVELS}"


# Coefficient file
COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_NAME_BIN=rtcoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_NAME}
rm -f ${COEF_NAME_BIN}
if [ -s  $DATART/${COEF_NAME} ] ; then
  ln -s $DATART/${COEF_NAME} ${COEF_NAME}
elif [ -s  $DATART/${COEF_NAME_BIN} ] ; then
  ln -s $DATART/${COEF_NAME_BIN} ${COEF_NAME_BIN}
fi


$BINDIR/example_fwd.exe << EOF
${PLATFORM_N}   , platform name
${SATID}        , satellite number (NOAAxx = xx)
${INST_N}       , instrument number
${SURF_TYPE}    , Surface type (0=land, 1=sea, 2=ice/snow)
${WATER_TYPE}   , Water type (0=fresh water, 1=ocean water)
${NLEVELS}      , number of levels
${ZENANGLE}     , Zenith angle
 1, 1, 0.0      , channel 1, valid, emissivity=0.0
 2, 1, 0.0      , channel 2, valid, emissivity=0.0
 3, 1, 0.0      , channel 3, valid, emissivity=0.0
EOF

mv output_example_fwd.dat output_example_fwd.dat.${ARCH}
diff -biw output_example_fwd.dat.${ARCH} \
  ${REF_TEST}/output_example_fwd.dat > diff_example_fwd.${ARCH}

echo 
echo "Output is in the file ${TEST}/output_example_fwd.dat.${ARCH}"
echo

rm -f ${COEF_NAME} ${COEF_NAME_BIN}

cd $CWD

exit
