#!/bin/sh
# Script to run the EXAMPLE_PC_FWD code
#
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# J Hocking Oct 2010

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######
# Relative to the rttov_test directory:
TEST=./test_example_pc_fwd.1

# Relative to the rttov_test/${TEST} directory:
BINDIR=../../$BIN
DATART=../../rtcoef_rttov10/rttov9pred101L
DATAPC=../../rtcoef_rttov10/pc
REF_TEST=../test_example_pc_fwd.2
###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


CWD=`pwd`
cd $TEST

for TEST_NUMBER in 01 02 
do

  case ${TEST_NUMBER} in
  
    '01') PLATFORM=eos ; INST=airs ; SATID=2 ;;
    
    '02') PLATFORM=metop ; INST=iasi ; SATID=2 ;;
  
  esac

  # Coefficient files
  COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
  COEF_NAME_BIN=rtcoef_${PLATFORM}_${SATID}_${INST}.bin
  rm -f ${COEF_NAME}
  rm -f ${COEF_NAME_BIN}
  if [ -s  $DATART/${COEF_NAME} ] ; then
    ln -s $DATART/${COEF_NAME} ${COEF_NAME}
  elif [ -s  $DATART/${COEF_NAME_BIN} ] ; then
    ln -s $DATART/${COEF_NAME_BIN} ${COEF_NAME_BIN}
  fi
  
  PC_COEF_NAME=pccoef_${PLATFORM}_${SATID}_${INST}.dat
  PC_COEF_NAME_BIN=pccoef_${PLATFORM}_${SATID}_${INST}.bin
  rm -f ${PC_COEF_NAME}
  rm -f ${PC_COEF_NAME_BIN}
  if [ -s  $DATAPC/${PC_COEF_NAME} ] ; then
    ln -s $DATAPC/${PC_COEF_NAME} ${PC_COEF_NAME}
  elif [ -s  $DATAPC/${PC_COEF_NAME_BIN} ] ; then
    ln -s $DATAPC/${PC_COEF_NAME_BIN} ${PC_COEF_NAME_BIN}
  fi
  
  
  $BINDIR/example_pc_fwd.exe << EOF
    $INST
EOF
  
  mv output_example_pc_fwd_${INST}.dat output_example_pc_fwd_${INST}.dat.${ARCH}
  diff -biw output_example_pc_fwd_${INST}.dat.${ARCH} \
    ${REF_TEST}/output_example_pc_fwd_${INST}.dat > diff_example_pc_fwd_${INST}.${ARCH}
  
  echo 
  echo "Output is in the file ${TEST}/output_example_pc_fwd_${INST}.dat.${ARCH}"
  echo
  
  rm -f ${COEF_NAME} ${COEF_NAME_BIN}
  rm -f ${PC_COEF_NAME} ${PC_COEF_NAME_BIN}

done

cd $CWD

exit
