#!/bin/sh
#
# MW emissivity atlas test control script
#
# 16/07/2010   Created (P. Brunel, J. Hocking)
#

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######

# Relative to the rttov_test/test_emisatlas.1 directory:

DATART=../../rtcoef_rttov10/rttov7pred51L
DATAATLAS=../../emis_data
EXEC=../../$BIN/rttov_cnrmmwatlas_test.exe
REF_TEST=../test_emisatlas.2

# Relative to the rttov_test directory:

TEST=./test_emisatlas.1

###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


set -x

MONTH=06

TEST_INPUT_FILE="rttov_cnrmmwatlas_test_input"

if [ ! -d $TEST ]; then
  echo 'Test directory required, containing profiles_mw'
  echo 'Also, make sure to run directly from the rttov_test directory'
  exit
fi

cwd=`pwd`
cd $TEST

PLATFORM=noaa
SATID=15
INST=amsua
TEST_NUMBER=01

cat  > $TEST_INPUT_FILE  <<EOF
1 15 3      # Satellite series, ID, sensor
$DATAATLAS
$MONTH      # Month for which to load emissivity data
650         # number of profiles
85, -0.20   # latitude start, latitude step
20,  0.00   # longitude start, longitude step
 0,  0.08   # Zenith angle (degrees)
6           # number of channels
1,2,3,4,7,15  # list of channels
EOF

  
COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
rm -f $COEF_NAME
if [ -s  $DATART/$COEF_NAME ] ; then
  ln -s $DATART/$COEF_NAME $COEF_NAME
else
  echo "RTCOEF tables not found"
  cd $cwd
  exit
fi

if [ ! -d $DATAATLAS ] ; then
  echo "Atlas emissivity directory not found"
  cd $cwd
  exit
fi
  

$EXEC < $TEST_INPUT_FILE

mv output_cnrmmwatlas.ascii  output_cnrmmwatlas.$TEST_NUMBER.$ARCH

if [ -d $REF_TEST ] ; then
  diff -biw output_cnrmmwatlas.$TEST_NUMBER.$ARCH \
    $REF_TEST/output_cnrmmwatlas.$TEST_NUMBER > diff_cnrmmwatlas.$TEST_NUMBER.$ARCH
      
    echo 
    echo "Output is in the file ${TEST}/output_cnrmmwatlas.$TEST_NUMBER.$ARCH"
    echo
    
else
  echo 'Reference test directory required, containing output_cnrmmwatlas.nn'
fi

rm -f $TEST_INPUT_FILE
rm -f $COEF_NAME


cd $cwd

exit
