#!/bin/sh

# User test

# Test performance of RTTOV through use of MULT and NTIMES parameters.
# Tests a variety of instruments on various profiles (51- and 101-level, and 
# cloudy/aerosol-affected).
# The REFRACTION, SOLAR, and APPLY_REG_LIMITS flags are tested.
# The use of multiple threads is also tested.

# No reference (.2 directory) needed for this test.

if [ "x$ARCH" = "x" ]
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
fi

set -x

SESSION_ONE=test_cpu_one
SESSION_TWO=test_cpu_two

OPTS=$*
WHAT="PRINT=0"
COEF="COEF_EXTRACT=1 COEF_FORMAT=UNFORMATTED"

./rttov_test.pl $WHAT $COEF ARCH=$ARCH $OPTS -- << EOF
  TEST_LIST=seviri/01    MULT=50    NTIMES=800   SESSION=$SESSION_ONE DIRECT=1 NTHREADS=4
  TEST_LIST=seviri/01    MULT=10    NTIMES=4000  SESSION=$SESSION_TWO DIRECT=1 NTHREADS=4
  TEST_LIST=amsua/div52  MULT=20    NTIMES=10    SESSION=$SESSION_ONE DIRECT=1 TL=1 AD=1 K=1 APPLY_REG_LIMITS=1
  TEST_LIST=amsua/div52             NTIMES=200   SESSION=$SESSION_TWO DIRECT=1 TL=1 AD=1 K=1 APPLY_REG_LIMITS=1
  TEST_LIST=seviri/div52 MULT=20    NTIMES=10    SESSION=$SESSION_ONE DIRECT=1 TL=1 AD=1 K=1 APPLY_REG_LIMITS=1
  TEST_LIST=seviri/div52            NTIMES=200   SESSION=$SESSION_TWO DIRECT=1 TL=1 AD=1 K=1 APPLY_REG_LIMITS=1
  TEST_LIST=hirs/21      MULT=10    NTIMES=100   SESSION=$SESSION_ONE DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1
  TEST_LIST=hirs/21      MULT=10    NTIMES=100   SESSION=$SESSION_TWO DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1 NTHREADS=4
  TEST_LIST=airs/01      MULT=50    NTIMES=10    SESSION=$SESSION_ONE DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1 SOLAR=1
  TEST_LIST=airs/01                 NTIMES=500   SESSION=$SESSION_TWO DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1 SOLAR=1
  TEST_LIST=airs/41      MULT=10    NTIMES=50    SESSION=$SESSION_ONE DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1 SOLAR=1
  TEST_LIST=airs/41                 NTIMES=500   SESSION=$SESSION_TWO DIRECT=1 TL=1 AD=1 K=1 REFRACTION=1 SOLAR=1
  TEST_LIST=iasi/21      MULT=50    NTIMES=100   SESSION=$SESSION_ONE DIRECT=1
  TEST_LIST=iasi/21      MULT=10    NTIMES=500   SESSION=$SESSION_TWO DIRECT=1
  TEST_LIST=iasi/41      MULT=20    NTIMES=25    SESSION=$SESSION_ONE DIRECT=1
  TEST_LIST=iasi/41                 NTIMES=500   SESSION=$SESSION_TWO DIRECT=1
  TEST_LIST=iasi/mocpu11 MULT=50    NTIMES=100   SESSION=$SESSION_ONE DIRECT=1
  TEST_LIST=iasi/mocpu11 MULT=10    NTIMES=500   SESSION=$SESSION_TWO DIRECT=1
  TEST_LIST=iasi/mocpu12 MULT=50    NTIMES=100   SESSION=$SESSION_ONE DIRECT=1
  TEST_LIST=iasi/mocpu12 MULT=10    NTIMES=500   SESSION=$SESSION_TWO DIRECT=1
EOF
