#!/bin/sh
#
# IR emissivity atlas test control script
#
# 14/06/2010   Created (J. Hocking)
#

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######

# Relative to the rttov_test/test_emisatlas.1 directory:
DATART=../../rtcoef_rttov10/rttov7pred51L
DATAATLAS=../../emis_data
EXEC=../../$BIN/rttov_iratlas_test.exe
REF_TEST=../test_emisatlas.2

# Relative to the rttov_test directory:
TEST=./test_emisatlas.1

###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


set -x

MONTH=08

ATLAS_EMIS_FILE="UWirbfemis_V2.1_0.1deg_2007${MONTH}_mask.nc"
ATLAS_COV_FILE="UWiremis_hsremis_covmat_V1.0_deg0.5_month${MONTH}_mask.nc"
ATLAS_EIGVEC_FILE="UWiremis_labeigvects.nc"

TEST_INPUT_FILE="rttov_iratlas_test_input"

if [ ! -d $TEST ]; then
  echo 'Test directory required, containing profiles_ir'
  echo 'Also, make sure to run directly from the rttov_test directory'
  exit
fi

cwd=`pwd`
cd $TEST

for TEST_NUMBER in 01 02 
# for TEST_NUMBER in 01 
do

  case $TEST_NUMBER in
  
    '01') PLATFORM=eos ; SATID=1 ; INST=modis
   
    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH                                    # Month for which to load emissivity data
9  1 13                                   # Satellite series, ID, sensor
16                                        # Number of channels
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16    # Channel list
$DATAATLAS
EOF

    ;;

    '02') PLATFORM=metop ; SATID=2 ; INST=iasi

    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH
10  2 16
19
93 137 185 237 285 349 413 1013 1533 2289 2877 3357 6181 6273 6381 6525 6865 7469 8105
$DATAATLAS
EOF
   
    ;;    
  esac
  
  COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
  rm -f $COEF_NAME
  if [ -s  $DATART/$COEF_NAME ] ; then
    ln -s $DATART/$COEF_NAME $COEF_NAME
  else
    echo "RTCOEF tables not found"
    cd $cwd
    exit
  fi

  if [ ! -s $DATAATLAS/$ATLAS_EMIS_FILE ] ; then
    echo "Atlas emissivity file not found"
    cd $cwd
    exit
  fi
  
  if [ ! -s $DATAATLAS/$ATLAS_COV_FILE ] ; then
    echo "Atlas covariance file not found"
    cd $cwd
    exit
  fi

  if [ ! -s $DATAATLAS/$ATLAS_EIGVEC_FILE ] ; then
    echo "Atlas eigenvector file not found"
    cd $cwd
    exit
  fi
  
  $EXEC < $TEST_INPUT_FILE

  mv output_iratlas.ascii  output_iratlas.$TEST_NUMBER.$ARCH

  if [ -d $REF_TEST ] ; then
    diff -biw output_iratlas.$TEST_NUMBER.$ARCH \
      $REF_TEST/output_iratlas.$TEST_NUMBER > diff_iratlas.$TEST_NUMBER.$ARCH
      
    echo 
    echo "Output is in the file ${TEST}/output_iratlas.$TEST_NUMBER.$ARCH"
    echo
    
  else
    echo 'Reference test directory required, containing output_iratlas.nn'
  fi

  rm -f $TEST_INPUT_FILE
  rm -f $COEF_NAME

done

cd $cwd

exit
