#!/bin/sh
#
# MW emissivity atlas test control script
#
# 14/06/2010   Created (J. Hocking)
#

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######

# Relative to the rttov_test/test_emisatlas.1 directory:
DATART=../../rtcoef_rttov10/rttov7pred51L
DATAATLAS=../../emis_data
EXEC=../../$BIN/rttov_mwatlas_test.exe
REF_TEST=../test_emisatlas.2

# Relative to the rttov_test directory:
TEST=./test_emisatlas.1

###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


set -x

MONTH=08

ATLAS_EMIS_FILE="ssmi_mean_emis_climato_${MONTH}_cov_interpol"
ATLAS_COR_FILE="correlations"

TEST_INPUT_FILE="rttov_mwatlas_test_input"

if [ ! -d $TEST ]; then
  echo 'Test directory required, containing profiles_mw'
  echo 'Also, make sure to run directly from the rttov_test directory'
  exit
fi

cwd=`pwd`
cd $TEST

for TEST_NUMBER in 01 02 
# for TEST_NUMBER in 01 
do

  case $TEST_NUMBER in
  
    '01') PLATFORM=noaa ; SATID=15 ; INST=amsua
   
    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH     # Month for which to load emissivity data
1 15 3     # Satellite series, ID, sensor
23         # Zenith angle (degrees)
$DATAATLAS
EOF

    ;;

    '02') PLATFORM=dmsp ; SATID=13 ; INST=ssmi

    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH     # Month for which to load emissivity data
2  13 6    # Satellite series, ID, sensor
57         # Zenith angle (degrees)
$DATAATLAS
EOF
   
    ;;    
  esac
  
  COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
  rm -f $COEF_NAME
  if [ -s  $DATART/$COEF_NAME ] ; then
    ln -s $DATART/$COEF_NAME $COEF_NAME
  else
    echo "RTCOEF tables not found"
    cd $cwd
    exit
  fi

  if [ ! -s $DATAATLAS/$ATLAS_EMIS_FILE ] ; then
    echo "Atlas emissivity file not found"
    cd $cwd
    exit
  fi
  
  if [ ! -s $DATAATLAS/$ATLAS_COR_FILE ] ; then
    echo "Atlas correlations file not found"
    cd $cwd
    exit
  fi

  $EXEC < $TEST_INPUT_FILE

  mv output_mwatlas.ascii  output_mwatlas.$TEST_NUMBER.$ARCH

  if [ -d $REF_TEST ] ; then
    diff -biw output_mwatlas.$TEST_NUMBER.$ARCH \
      $REF_TEST/output_mwatlas.$TEST_NUMBER > diff_mwatlas.$TEST_NUMBER.$ARCH
      
    echo 
    echo "Output is in the file ${TEST}/output_mwatlas.$TEST_NUMBER.$ARCH"
    echo
    
  else
    echo 'Reference test directory required, containing output_mwatlas.nn'
  fi

  rm -f $TEST_INPUT_FILE
  rm -f $COEF_NAME

done

cd $cwd

exit
