#!/bin/sh
#
# RTTOV_SCATT tests control script
#
# This test requires the (~1GB) Mie Tables to be available
# and placed in a directory pointed to by DATAMIE
# 
# 13/12/2007   Alan Geer   First version
# 24/02/2010   Alan Geer   Temporary version for RTTOV-10 - only N15 AMSU-A
# 30/07/2010   J Hocking   Add SSM/I test
#

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######

# Relative to the rttov_test/test_rttovscatt.1 directory:
DATART=../../rtcoef_rttov10/rttov7pred51L
DATAMIE=../../rtcoef_rttov10/mietable
EXEC=../../$BIN/rttovscatt_test.exe
REF_TEST=../test_rttovscatt.2

# Relative to the rttov_test directory:
TEST=./test_rttovscatt.1

###################################################

if [ "x$ARCH" = "x" ];
then
  ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
  if [ "x$ARCH" = "x" ];
  then
    echo 'Please supply ARCH'
    exit 1
  fi
fi


set -x

if [ ! -d ${TEST} ]; then
  echo 'Test directory required, containing profiles2_fmt'
  echo 'Also, make sure to run directly from the rttov_test directory'
  exit
fi

cwd=`pwd`
cd $TEST

TEST_INPUT_FILE=rttovscatt_test_input

for TEST_NUMBER in 01 02 
do
   
#  export PROGINF=DETAIL
#  export DR_HOOK=1
#  export DR_HOOK_OPT='hpmprof'
#  export OMP_NUM_THREADS=1

  case ${TEST_NUMBER} in
  
    '01') PLATFORM=dmsp ; SATID=13 ; INST=ssmi
   
    cat  > ${TEST_INPUT_FILE}  <<EOF
2   # Satellite series
13  # Satellite ID
6   # Satellite sensor
57  # zenith angle (degrees)
EOF

    ;;

    '02') PLATFORM=noaa ; SATID=15 ; INST=amsua

    cat  > ${TEST_INPUT_FILE}  <<EOF
1   # Satellite series
15  # Satellite ID
3   # Satellite sensor
23  # zenith angle (degrees)
EOF
   
    ;;    
  esac
  
  COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
  rm -f ${COEF_NAME}
  if [ -s  $DATART/${COEF_NAME} ] ; then
    ln -s $DATART/${COEF_NAME} ${COEF_NAME}
  else
    echo "RTCOEF tables not found"
    cd $cwd
    exit
  fi

  MIETABLE_NAME=mietable_${PLATFORM}_${INST}.dat
  rm -f ${MIETABLE_NAME}
  if [ -s  $DATAMIE/${MIETABLE_NAME} ] ; then
    ln -s $DATAMIE/${MIETABLE_NAME} ${MIETABLE_NAME}
  else
    echo "Mie tables not found"
    cd $cwd
    exit
  fi

  $EXEC < ${TEST_INPUT_FILE}

#  mv drhook.prof.1      drhook.prof.${TEST_NUMBER}.rttov10_scatt
  mv outputscatt_full.ascii  output_full.${TEST_NUMBER}.rttov10_scatt.${ARCH}
  mv outputscatt.ascii       output.${TEST_NUMBER}.rttov10_scatt.${ARCH}

  if [ -d ${REF_TEST} ] ; then
    diff -biw output.${TEST_NUMBER}.rttov10_scatt.${ARCH} \
      $REF_TEST/output.${TEST_NUMBER}.rttov10_scatt > diff.${TEST_NUMBER}.${ARCH}
      
    echo 
    echo "Output is in the file ${TEST}/output.${TEST_NUMBER}.rttov10_scatt.${ARCH}"
    echo

  else
    echo 'Reference test directory required, containing output.nn.rttov10_scatt'
  fi

  rm -f ${TEST_INPUT_FILE}
done

cd $cwd

exit
