!     Compute aerosols density for 10 climatological profiles           

      Program aerden

! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!     Description:
!     To set up profile-dependent variables for subsequent
!     rt calculations by other subroutines of RTIASI.
!
!
!     History:
!     Version      Date        Comment
!     1           04/10/2007    Marco Matricardi. ECMWF.
!
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".
!

      IMPLICIT NONE

      INTEGER, PARAMETER :: fp = SELECTED_REAL_KIND(13,300)

!       Local scalars:
      INTEGER :: I,II,J,IR
      INTEGER :: ILAY

      REAL(fp)    :: SCALFAC
      REAL(fp)    :: RLH
      REAL(fp)    :: HFACSL1      
      REAL(fp)    :: HFACSL2      
      REAL(fp)    :: HFACFT1      
      REAL(fp)    :: HFACFT2      
      REAL(fp)    :: DFLAT
      REAL(fp)    :: FAC
      REAL(fp)    :: ETA
      REAL(fp)    :: BETA
      REAL(fp)    :: GRAVL
      REAL(fp)    :: GRAVH
      REAL(fp)    :: LAT
      REAL(fp)    :: LATR
      REAL(fp)    :: ZERHT
      REAL(fp)    :: TS
      REAL(fp)    :: DTOR
      REAL(fp)    :: Q_MIXRATIO_TO_PPMV
      REAL(fp)    :: NDENSST
      REAL(fp)    :: FLATT            ! Flattening of the Earth
      REAL(fp)    :: EQRAD            ! Equatorial radius of the earth          [km]
      REAL(fp)    :: REARTH           ! Radius of earth at latitude LAT         [km]
      REAL(fp)    :: OMEGA            ! Angular velocity of the Earth        [rad/s]
      REAL(fp)    :: GRAVE            ! Normal gravity at equator            [m/s^2]
      REAL(fp)    :: RGC              ! Molar gas constant               [J/(K*mol)]
      REAL(fp)    :: AIRMW            ! Dry air molecular weight
      REAL(fp)    :: H2OMW            ! Water molecular weight
      REAL(fp)    :: TEMP    (101)
      REAL(fp)    :: TEMPP   (101)
      REAL(fp)    :: WMIX    (101)
      REAL(fp)    :: Q       (101)
      REAL(fp)    :: PRES    (101)
      REAL(fp)    :: XPRES   (101)
      REAL(fp)    :: WATER   (101)
      REAL(fp)    :: DAIR    (101)
      REAL(fp)    :: DMAIR   (101)
      REAL(fp)    :: HGPL    (101)
      REAL(fp)    :: INT     (101)
      REAL(fp)    :: HL      (101)
      REAL(fp)    :: LTICK   (101)
      REAL(fp)    :: CHEIG   (10,3)
      REAL(fp)    :: ZFAC    (10,4)
      REAL(fp)    :: NDENS   (5,10)
      REAL(fp)    :: NDENSFT (3)
      REAL(fp)    :: ADENS   (101,10,10)
      REAL(fp)    :: HFAC    (101)
      INTEGER :: AFAC    (3)
      INTEGER :: CCOMP   (5,10)
      INTEGER :: CNUM    (10)
      INTEGER :: NLEVSF
      INTEGER :: NLEV
      

!-----End of header-------------------------------------------------------------

      DATA FLATT  /3.3528107E-3_fp/  ! Geodetic Reference System 1980
      DATA EQRAD  /6378.137_fp    /  ! Geodetic Reference System 1980
      DATA OMEGA  /7292115E-11_fp /  ! Geodetic Reference System 1980
      DATA GRAVE  /9.7803267715_fp/  ! Geodetic Reference System 1980
      DATA RGC    /8.314472_fp    /  ! CODATA recommended value  1998
      DATA TS     /273.15_fp      /
      DATA DTOR   /0.0174532925_fp/
      DATA AIRMW  /28.964_fp      /
      DATA H2OMW  /18.0115_fp     /
      DATA Q_MIXRATIO_TO_PPMV/1.60771704E+6_fp/

      DATA CHEIG/02_fp,02_fp,02_fp,02_fp,06_fp,02_fp,02_fp,02_fp,02_fp,10_fp,  &
                 12_fp,12_fp,12_fp,12_fp,12_fp,12_fp,12_fp,12_fp,12_fp,12_fp,  &
                 35_fp,35_fp,35_fp,35_fp,35_fp,35_fp,35_fp,35_fp,35_fp,35_fp/

      DATA ZFAC /8_fp ,8_fp ,8_fp ,8_fp ,2_fp ,1_fp ,1_fp ,1_fp ,99_fp,8_fp,   &
                 8_fp ,8_fp ,8_fp ,8_fp ,8_fp ,8_fp ,8_fp ,8_fp ,8_fp ,8_fp,   &
                 99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,  &
                 99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp,99_fp/

      DATA CCOMP/2,1,0,0,0,                                                    &
                 2,1,3,0,0,                                                    &
                 2,1,3,0,0,                                                    &
                 2,1,3,0,0,                                                    &
                 2,6,7,8,0,                                                    &
                 2,4,5,0,0,                                                    &
                 2,4,5,3,0,                                                    &
                 2,4,5,0,0,                                                    &
                 2,1,4,3,0,                                                    &
                 10,4,9,0,0/

      DATA NDENS/2600_fp , 0.15_fp   , 0._fp     , 0._fp   , 0._fp,            &
                 7000_fp , 0.4_fp    , 8300_fp   , 0._fp   , 0._fp,            &
                 15700_fp, 0.6_fp    , 34300_fp  , 0._fp   , 0._fp,            &
                 28000_fp, 1.5_fp    , 130000_fp , 0._fp   , 0._fp,            &
                 2000_fp , 269.5_fp  , 30.5_fp   , 0.142_fp, 0._fp,            &
                 1500_fp , 20_fp     , 3.2e-3_fp , 0._fp   , 0._fp,            &
                 3800_fp , 20_fp     , 3.2e-3_fp , 5180_fp , 0._fp,            &
                 590_fp  , 10_fp     , 1.3e-3_fp , 0._fp   , 0._fp,            &
                 1300_fp , 0.01_fp   , 1.9_fp    , 5300_fp , 0._fp,            &
                 42.9_fp , 0.47e-1_fp, 0.53e-2_fp, 0._fp   , 0._fp/

      DATA NDENSFT/438_fp,0.1241_fp,292_fp/

      DATA NDENSST/3_fp/


      DATA CNUM /2,3,3,3,4,3,4,3,4,3/


      WRITE(6,*)'ENTER LATITUDE'
      READ (5,*)LAT
      WRITE(6,*)'ENTER ELEVATION'
      READ (5,*)ZERHT
      WRITE(6,*)'ENTER SURFACE LEVEL'
      READ (5,*)NLEVSF
      WRITE(6,*)'ENTER SCALING FACTOR'
      READ (5,*)SCALFAC

      OPEN(10,FILE='prof_aerosl_cl.dat')

      OPEN(1,FILE='plevs.dat')

      READ(1,*)NLEV
      READ(1,*)(XPRES(I),I=1,NLEV)

      OPEN(2,FILE='prof.dat')

      READ(2,*)(TEMP(I),I=1,NLEV)
      READ(2,*)(Q(I),   I=1,NLEV)

!       WMIX(1:NLEV)=(exp(q(1:NLEV)) / 1000.) * q_mixratio_to_ppmv
      WMIX(1:NLEV)=q(1:NLEV)    ! Input units are now ppmv

      ADENS(:,:,:)=0.



!---------Compute tickness of the layer-----------------------------------------

!-------1.  Caculate the earth's radius at latitude LAT assuming the-----------
!       Earth is an ellipsoid of revolution                                    |
!------------------------------------------------------------------------------

        DFLAT  = (1.0 - FLATT)**2
        LATR   = DTOR*ABS(LAT)
        REARTH = SQRT(EQRAD**2*DFLAT/(SIN(LATR)**2 + DFLAT*COS(LATR)**2))


!-------2.  The valute of earth's gravity at surface at latitude LAT is--------
!       computed using the international gravity formula.                      |
!------------------------------------------------------------------------------

        FAC   = (omega**2*(EQRAD*1000))/(GRAVE)
        BETA  = 5*FAC/2-FLATT-17*FAC*FLATT/14
        ETA   = FLATT*(5*FAC-FLATT)/8
        GRAVL = GRAVE*(1+BETA*(SIN(DTOR*LAT))**2+ETA*(SIN(2*DTOR*LAT))**2)


!-------3.  The value of the gravity as a function of altitude H can be--------
!       expressed using the inverse-square law of gravitation:                 |
!                                                                              |
!       GRAVL(H)=GRAVL*(REARTH/(H+REARTH))**2=                                 |
!                GRAVL*(1-2*H/REARTH+3*H**2/REARTH**2+terms of higher order)   |
!                                                                              |
!       If we eliminate the second and higher order terms we can write:        |
!                                                                              |
!       GRAVL(H)=GRAVL-2*GRAVL*H/REARTH=GRAVL-GRAVH*H                          |
!------------------------------------------------------------------------------

        GRAVH = 2.0E-3*GRAVL/REARTH


!-------4.  Unpack input profile------------------------------------------------

        DO I=1,NLEV
            PRES(I) =XPRES(NLEV-I+1)
            WATER(I)=WMIX (NLEV-I+1)
            TEMPP(I)=TEMP (NLEV-I+1)
        ENDDO


!-------5.  Calculate density for dry air (ideal gas)---------------------------

        DO IR=1,NLEV
           DAIR(IR)=1.E+02*PRES(IR)*AIRMW/(1000*RGC*TEMPP(IR))
        END DO


!-------6.  The density of dry air is adjusted to account for the presence ----
!       of water vapour by introducing a scaling factor (i.e. the              |
!       temperature is replaced by the virtual temperature).The density        |
!       for moist air is thus obtained. It is assumed that the partial         |
!       pressure for water vapour can be written as PRES*WATER*1.E-6           |
!------------------------------------------------------------------------------

        DO IR=1,NLEV
          DMAIR(IR)= DAIR(IR)*(AIRMW*(1.0E6 - WATER(IR))+H2OMW*WATER(IR))/  &
                                     (1.0E6*AIRMW)
        ENDDO


!-------7.  Set up the height of the surface pressure level---------------------

        HGPL(NLEVSF) = ZERHT


!-------8  Compute height of pressure levels:levels above surface. ------------
!       The height of pressure levels H is obtained by integrating             |
!       the hydrostatic equation dPRES=-GRAVL(H)*DMAIR*dH between              |
!       two adjacent pressure levels                                           |
!                                                                              |
!           -P2         - H2                                                   |
!          |           |                                                       |
!          | dP/DMAIR= | GRAVL(H)*dH                                           |
!          |           |                                                       |
!         -  P1       -   H1                                                   |
!                                                                              |
!       The integration id DMAIR is carried out assuming this                  |
!       quantity varies linearly with pressure. The integral is                |
!       then computed applying the trapezoidal formula.                        |
!------------------------------------------------------------------------------

        DO IR = NLEVSF+1,NLEV
          INT(IR)     =- 0.5*(PRES(IR)-PRES(IR-1))*                          &
                         (1/DMAIR(IR)+1/DMAIR(IR-1))
          INT(IR)     =INT(IR)*1.0E2            ! Integral in units [m2.s-1]
          RLH     =GRAVL/GRAVH
          HL(IR)=1.0E3*HGPL(IR-1)
          HGPL(IR)=(RLH -  SQRT(RLH**2 - HL(IR)*(2.0*RLH - HL(IR))     - &
                     2.0*INT(IR)/GRAVH))*1.0E-3
        ENDDO


!-------8.1  Compute height of pressure levels:levels below surface-------------

        DO IR =NLEVSF-1,1,-1
          INT(IR)     =0.5*(PRES(IR+1)-PRES(IR-1+1))*                        &
                       (1/DMAIR(IR+1)+1/DMAIR(IR))
          INT(IR)     =INT(IR)*1.0E2            ! Integral in units [m2.s-1]
          RLH     =GRAVL/GRAVH
          HL(IR)      =1.0E3*HGPL(IR+1)
          HGPL(IR)=(RLH -  SQRT(RLH**2 - HL(IR)*(2.0*RLH - HL(IR))     - &
                     2.0*INT(IR)/GRAVH))*1.0E-3

        ENDDO



!---------Compute tickness of the layer-----------------------------------------

          DO ILAY=1,NLEV-1
            LTICK(ILAY)=HGPL(ILAY+1)-HGPL(ILAY)
          ENDDO
!-------------------------------------------------------------------------------



!---------Compute aerosol number density in each layer for a given -------------
!         climatological composition:
!
!         J=1  -->Continental clean
!         J=2  -->Continental average
!         J=3  -->Continental polluted
!         J=4  -->Urban               
!         J=5  -->Desert             
!         J=6  -->Maritime clean    
!         J=7  -->Maritime polluted
!         J=8  -->Maritime tropical
!         J=9  -->Arctic
!         J=10 -->Antarctic

          DO J=1,10


!-----------Nearest layer-------------------------------------------------------

            DO I=1,3
              DO ILAY=1,NLEV
                IF(HGPL(ILAY)>CHEIG(J,I))THEN
                  AFAC(I)=ILAY
                  EXIT
                ENDIF
              ENDDO
            ENDDO
!-------------------------------------------------------------------------------



!-----------Compute the average number density in each layer:planetary layer----


            DO ILAY=1,AFAC(1)-2
              IF(ZFAC(J,1   ) /= 99)THEN
                HFAC(ILAY)=ZFAC(J,1)                                         * &
                           (EXP(-HGPL(ILAY)/ZFAC(J,1))                       - &
                           EXP(-HGPL(ILAY+1)/ZFAC(J,1)))
              ELSE IF(ZFAC(J,1   ) == 99)THEN
                HFAC(ILAY)=LTICK(ILAY)
              ENDIF
                DO I=1,CNUM(J)
                  II   =CCOMP(I,J)
                  ADENS(ILAY,J,II)=SCALFAC*NDENS(I,J)*HFAC(ILAY)
                ENDDO
            ENDDO

            IF(ZFAC(J,1   ) /= 99)THEN
              HFACSL1     =ZFAC(J,1)                                         * &
                          (EXP(-HGPL(ILAY)/ZFAC(J,1))                        - &
                          EXP(-CHEIG(J,1)/ZFAC(J,1)))
            ELSE IF(ZFAC(J,1   ) == 99)THEN
              HFACSL1     =CHEIG(J,1)-HGPL(ILAY)
            ENDIF
            DO I=1,CNUM(J)
              II   =CCOMP(I,J)
              ADENS(ILAY,J,II)=SCALFAC*NDENS(I,J)*HFACSL1
            ENDDO

            IF(ZFAC(J,2   ) /= 99)THEN
              HFACSL2=ZFAC(J,2)                                              * &
                      (EXP(-CHEIG(J,1)/ZFAC(J,2))                            - &                       
                      EXP(-HGPL(ILAY+1)/ZFAC(J,2)))
            ELSE IF(ZFAC(J,2   ) == 99)THEN
              HFACSL2     =(HGPL(ILAY+1)-CHEIG(J,1))
            ENDIF
            DO I=1,CNUM(3)
              II   =CCOMP(I,3)
              ADENS(ILAY,J,II)=SCALFAC*NDENSFT(I)*HFACSL2
            ENDDO
!-------------------------------------------------------------------------------



!-----------Compute the average number density in each layer:free troposphere---

            DO ILAY=AFAC(1),AFAC(2)-2
              IF(ZFAC(J,2   ) /= 99)THEN
                HFAC(ILAY)=ZFAC(J,2)                                         * &
                           (EXP(-HGPL(ILAY)/ZFAC(J,2))                       - &
                           EXP(-HGPL(ILAY+1)/ZFAC(J,2   )))
              ELSE IF(ZFAC(J,2   ) == 99)THEN
                HFAC(ILAY)=LTICK(ILAY)
              ENDIF
              DO I=1,CNUM(3)
                II   =CCOMP(I,3)
                ADENS(ILAY,J,II)=SCALFAC*NDENSFT(I)*HFAC(ILAY)
              ENDDO
            ENDDO

            IF(ZFAC(J,2   ) /= 99)THEN
              HFACFT1     =ZFAC(J,2)                                         * &
                           (EXP(-HGPL(ILAY)/ZFAC(J,2))                       - &
                           EXP(-CHEIG(J,2)/ZFAC(J,2)))
            ELSE IF(ZFAC(J,2   ) == 99)THEN
              HFACFT1     =CHEIG(J,2)-HGPL(ILAY)
            ENDIF
            DO I=1,CNUM(3)
              II   =CCOMP(I,3)
              ADENS(ILAY,J,II)=SCALFAC*NDENSFT(I)*HFACFT1
            ENDDO

            IF(ZFAC(J,3   ) /= 99)THEN
              HFACFT2  =ZFAC(J,3)                                            * &
                        (EXP(-CHEIG(J,2)/ZFAC(J,3))                          - &
                        EXP(-HGPL(ILAY+1)/ZFAC(J,3)))
            ELSE IF(ZFAC(J,3   ) == 99)THEN
              HFACFT2     =(HGPL(ILAY+1)-CHEIG(J,2))
            ENDIF
            ADENS(ILAY,J,10)=SCALFAC*NDENSST*HFACFT2
!-------------------------------------------------------------------------------



!-----------Compute the average number density in each layer:stratosphere-------

            DO ILAY=AFAC(2),AFAC(3)-1
              IF(ZFAC(J,3   ) /= 99)THEN
                HFAC(ILAY)=ZFAC(J,3)                                         * &
                           (EXP(-HGPL(ILAY)/ZFAC(J,3))                       - &
                           EXP(-HGPL(ILAY+1)/ZFAC(J,3   )))
              ELSE IF(ZFAC(J,3   ) == 99)THEN
                HFAC(ILAY)=LTICK(ILAY)
              ENDIF
              ADENS(ILAY,J,10)=SCALFAC*NDENSST*HFAC(ILAY) !HFACFT2
            ENDDO

        do ii=nlev-1,1,-1
          write(10,'(11f11.3)')(ADENS(ii,j,i)/ltick(ii),i=1,10),0.
        enddo

      ENDDO

      END PROGRAM AERDEN 

