!
SUBROUTINE rttov_print_opts (opts, lu, text)
  ! Description:
  ! print the contents of the options structure
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       05/04/2011  New code 
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:

  ! Imported Type Definitions:
  USE rttov_types, ONLY :       &
        & rttov_options

  USE parkind1, ONLY : jpim

!INTF_OFF
USE rttov_global, ONLY : &
    error_unit
!INTF_ON

  IMPLICIT NONE

  ! subroutine arguments
  TYPE( rttov_options ), INTENT(IN)           :: opts  ! options
  INTEGER(KIND=jpim),    INTENT(IN), OPTIONAL :: lu    ! logical unit for print
  CHARACTER(LEN=*),      INTENT(IN), OPTIONAL :: text  ! additional text to print

!INTF_END
  INTEGER(KIND=jpim)  :: iu       ! logical unit for print
  CHARACTER(LEN=20)   :: tmp_text ! temporary string for formatting output

  iu = error_unit
  IF( PRESENT( lu ) ) iu = lu

  IF( PRESENT( text ) ) THEN
    WRITE(iu,'(/,a,a)') "RTTOV options structure: ",TRIM(text)
  ELSE
    WRITE(iu,'(/,a)') "RTTOV options structure"
  END IF
  WRITE(iu,'(2x,a,l1)')     "spacetop          ", opts%spacetop
  WRITE(iu,'(2x,a,l1)')     "lgradp            ", opts%lgradp
  WRITE(iu,'(2x,a,l1)')     "use_q2m           ", opts%use_q2m
  WRITE(iu,'(2x,a,l1)')     "switchrad         ", opts%switchrad
  WRITE(iu,'(2x,a,l1)')     "verbose           ", opts%verbose_checkinput_warnings
  WRITE(iu,'(2x,a,l1)')     "do_checkinput     ", opts%do_checkinput
  WRITE(iu,'(2x,a,l1)')     "apply_reg_limits  ", opts%apply_reg_limits
  WRITE(iu,'(a)',advance='yes')
  
  WRITE(iu,'(2x,a,l1)')     "addinterp         ", opts%addinterp
  WRITE(iu,'(2x,a,l1)')     "addrefrac         ", opts%addrefrac
  WRITE(iu,'(2x,a,l1)')     "addsolar          ", opts%addsolar
  WRITE(iu,'(a)',advance='yes')
  
  WRITE(iu,'(2x,a,l1)')     "ozone_data        ", opts%ozone_data
  WRITE(iu,'(2x,a,l1)')     "co2_data          ", opts%co2_data
  WRITE(iu,'(2x,a,l1)')     "n2o_data          ", opts%n2o_data
  WRITE(iu,'(2x,a,l1)')     "co_data           ", opts%co_data
  WRITE(iu,'(2x,a,l1)')     "ch4_data          ", opts%ch4_data
  WRITE(iu,'(2x,a,l1)')     "clw_data          ", opts%clw_data
  WRITE(iu,'(a)',advance='yes')
  
  WRITE(tmp_text,'(i3)') opts%fastem_version
  WRITE(iu,'(2x,a,a)')      "fastem_version    ", TRIM(ADJUSTL(tmp_text))
  WRITE(iu,'(a)',advance='yes')
  
  WRITE(iu,'(2x,a,l1)')     "addclouds         ", opts%addclouds
  WRITE(iu,'(2x,a,l1)')     "addaerosl         ", opts%addaerosl
  WRITE(tmp_text,'(f10.4)') opts%cldstr_threshold
  WRITE(iu,'(2x,a,a)')      "cldstr_threshold  ", TRIM(ADJUSTL(tmp_text))
  WRITE(iu,'(a)',advance='yes')
  
  WRITE(iu,'(2x,a,l1)')     "addpc             ", opts%addpc
  WRITE(iu,'(2x,a,l1)')     "addradrec         ", opts%addradrec
  WRITE(tmp_text,'(i3)') opts%ipcreg
  WRITE(iu,'(2x,a,a)')      "ipcreg            ", TRIM(ADJUSTL(tmp_text))

END SUBROUTINE rttov_print_opts
