!
Subroutine rttov_print_profile (profile, lu, text)
  ! Description:
  ! print a profile structure
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       21/12/2007  New code 
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:

  ! Imported Type Definitions:
  Use rttov_types, Only :       &
        & profile_Type

  Use rttov_const, Only :       &
        & aer_name
  

  Use parkind1, Only : jpim

!INTF_OFF
Use rttov_global, Only : &
    error_unit
!INTF_ON

  Implicit None

  ! subroutine arguments
  Type( profile_Type ), Intent(in)          :: profile ! profile
  Integer(Kind=jpim),   Intent(in),optional :: lu      ! logical unit for print
  Character(Len=*),     Intent(in),optional :: text    ! text for print

!INTF_END
  Integer(Kind=jpim)  :: iu  ! logical unit for print
  Integer(kind=jpim)  :: l   ! level



  iu = error_unit
  If( Present( lu ) ) iu = lu

  If( Present( text ) ) Then 
    write(iu,'(/,a,a)') "RTTOV  profile structure: ",TRIM(text)
  Else
    write(iu,'(/,a)') "RTTOV  profile structure"
  End If
  write(iu,'(2x,a,a)')                      "id        ", TRIM(profile%id)
  write(iu,'(2x,a,i4.4,"/",i2.2,"/",i2.2)') "date      ", profile%date
  write(iu,'(2x,a,i2.2,":",i2.2,":",i2.2)') "time      ", profile%time
  write(iu,'(2x,a,i4)') "number of levels ", profile%nlevels
  write(iu,'(2x,a,i4)') "number of layers ", profile%nlayers
  write(iu,'(a)') "Viewing conditions "
  write(iu,'(2x,a,f10.3)') "zenangle     ", profile%zenangle
  write(iu,'(2x,a,f10.3)') "azangle      ", profile%azangle
  write(iu,'(2x,a,f10.3)') "sunzenangle  ", profile%sunzenangle
  write(iu,'(2x,a,f10.3)') "sunazangle   ", profile%sunazangle
  write(iu,'(2x,a,f10.3)') "elevation    ", profile%elevation
  write(iu,'(2x,a,f10.3)') "latitude     ", profile%latitude
  write(iu,'(2x,a,f10.3)') "longitude    ", profile%longitude
  write(iu,'(2x,a,e14.6)') "snow_frac (0 - 1)       ", profile%snow_frac
  write(iu,'(2x,a,e14.6)') "soil_moisture (m^3/m^3) ", profile%soil_moisture
  write(iu,'(2x,a,e14.6)') "Earth magnetic field   ", profile%be
  write(iu,'(2x,a,e14.6)') "cos of Earth magnetic field and wave propagation ", profile%cosbk

  write(iu,'(a)') "Black body cloud"
  write(iu,'(2x,a,e14.6)') "cloud top pressure  (hPa) ",  profile%ctp
  write(iu,'(2x,a,e14.6)') "cloud fraction (0 - 1)    ",  profile%cfraction    

  write(iu,'(a)') "Skin parameters"
  write(iu,'(2x,a,i4)')    "surface type 0=land, 1=sea, 2=sea-ice    ", profile%skin%surftype
  write(iu,'(2x,a,i4)')    "water type  0=fresh water, 1=ocean water ", profile%skin%watertype
  write(iu,'(2x,a,e14.6)') "radiative skin temperature (K)           ", profile%skin%t
  write(iu,'(2x,a,e14.6)') "salinity (%o)                            ", profile%skin%salinity
  write(iu,'(2x,a)')       "land/sea-ice surface parameters for fastem"
  write(iu,'(10x,10f10.4)')  profile%skin%fastem(:)

  write(iu,'(a)') "2m air parameters"
  write(iu,'(2x,a,e14.6)') "surface pressure (hPa) ",profile%s2m%p
  write(iu,'(2x,a,e14.6)') "temperature (K)        ",profile%s2m%t
  write(iu,'(2x,a,e14.6)') "water vapour (ppmv)    ",profile%s2m%q
  write(iu,'(2x,a,e14.6)') "ozone (ppmv)           ",profile%s2m%o
  write(iu,'(2x,a,e14.6)') "U wind component (m/s) ",profile%s2m%u
  write(iu,'(2x,a,e14.6)') "V wind component (m/s) ",profile%s2m%v
  write(iu,'(2x,a,e14.6)') "Wind fetch (m)         ",profile%s2m%wfetc
 

  write(iu,'(/,a5,a9,1x,a14,1x,a14)',advance='no') "level", "Pressure", "Temp", "WV (ppmv)"
  If (Associated (profile%o3))  write(iu,'(1x,a14)',advance='no') "O3 (ppmv)"
  If (Associated (profile%co2)) write(iu,'(1x,a14)',advance='no') "CO2 (ppmv)"
  If (Associated (profile%n2o)) write(iu,'(1x,a14)',advance='no') "N2O (ppmv)"
  If (Associated (profile%co))  write(iu,'(1x,a14)',advance='no') "CO (ppmv)"
  If (Associated (profile%ch4)) write(iu,'(1x,a14)',advance='no') "CH4 (ppmv)"
  If (Associated (profile%clw)) write(iu,'(1x,a14)',advance='no') "CLW (ppmv)"
  write(iu,'(a)',advance='yes')

  do l = 1, profile%nlevels
   write(iu,'(i4,f12.4,e14.6,1x,e14.6)',advance='no') l, profile%p(l), profile%t(l), profile%q(l)
   If (Associated (profile%o3)) &
   write(iu,'(1x,e14.6)',advance='no') profile%o3(l)
   If (Associated (profile%co2)) &
   write(iu,'(1x,e14.6)',advance='no') profile%co2(l)
   If (Associated (profile%n2o)) &
   write(iu,'(1x,e14.6)',advance='no') profile%n2o(l)
   If (Associated (profile%co)) &
   write(iu,'(1x,e14.6)',advance='no') profile%co(l)
   If (Associated (profile%ch4)) &
   write(iu,'(1x,e14.6)',advance='no') profile%ch4(l)
   If (Associated (profile%clw)) &
   write(iu,'(1x,e14.6)',advance='no') profile%clw(l)
   write(iu,'(a)',advance='yes')
  end do

  if( Associated(profile%aerosols) ) then
    write(iu,'(/,a)') "Aerosols"
    write(iu,'(a5,1x,a21,2x,a)') "layer","Pressure  top  bottom",&
                       &"Aerosols mean number density in units of cm-3"
    write(iu,'(29x,11a14)') aer_name
    do l = 1, profile%nlayers
      write(iu,'(1x,i4,1x,f10.4,1x,f10.4,2x,11e14.6)') &
           & l, profile%p(l), profile%p(l+1), profile%aerosols(:,l)
    end do
  end if

  if( Associated(profile%cloud) ) then 
    write(iu,'(/,a)') "Clouds"
    write(iu,'(2x,a,i4)') "sheme iwc to eff diameter ", profile%idg
    write(iu,'(2x,a,i4)') "ice crystals shape        ", profile%ish
    write(iu,'(a5,1x,a21,a)') "layer", "Pressure  top  bottom", "  Cloud (g/m3)"
    do l = 1, profile%nlayers
      write(iu,'(1x,i4,1x,f10.4,1x,f10.4,2x,10e14.6)') l, profile%p(l), profile%p(l+1), profile%cloud(:,l)
      write(iu,'(22x,"cfrac",2x,10f14.6)')  profile%cfrac(:,l)
      write(iu,'(22x,"icede",2x,f14.6)')  profile%icede(l)
    end do
  end if
End Subroutine rttov_print_profile
