Program example_fwd
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2010, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !     TEST PROGRAM FOR RTTOV SUITE FORWARD MODEL ONLY
  !          RTTOV VERSION 10
  ! To run this program you must have the following files
  ! either resident in the same directory or set up as a
  ! symbolic link:
  !   prof.dat                      --  input profile
  !   rtcoef_platform_id_sensor.dat --  coefficient file to match
  !   the sensor you request in the input dialogue
  ! The script run_example_fwd.sh may be used to run this program.
  ! The output is generated in a file called example_fwd_output.dat.
  !
  !
  ! If the user wants to use this example to create his own
  ! program he will have to modify the code between
  ! comment lines of that kind:
  !     !================================
  !     !======Read =====start===========
  !          code to be modified
  !     !======Read ===== end ===========
  !     !================================
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !  1.0    27/04/2004   orginal (based on tstrad) P. Brunel
  !  1.1    09/08/2004   modified to allow for variable no. channels/per profile
  !                       R. Saunders
  !  1.2    13/04/2007   Modified for RTTOV-90
  !  1.3    31/07/2007   Modified for RTTOV-91 R Saunders
  !  1.4    11/10/2007   Parallel version P.Marguinaud
  !  2.0    25/06/2010   Modified for RTTOV-10 J Hocking
  !  2.1    23/11/2011   Updates for v10.2 J Hocking
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  Use rttov_const, Only :  &
       & errorstatus_success,&
       & errorstatus_warning,&
       & errorstatus_fatal  ,& 
       & q_mixratio_to_ppmv, &
       & platform_name,      &
       & inst_name

  Use rttov_types, Only :   &
       & rttov_options,     &
       & rttov_coefs,       &
       & profile_Type,      &
       & transmission_Type, &
       & radiance_Type,     &
       & rttov_chanprof

  Use parkind1, Only : jpim, jprb, jplm
  !
  Implicit None
  !
#ifdef _RTTOV_EXAMPLE_FWD_PARALLEL
#include "rttov_parallel_direct.interface"
#define rttov_direct rttov_parallel_direct
#else
#include "rttov_direct.interface"
#endif
#include "rttov_setup.interface"
#include "rttov_copy_prof.interface"
#include "rttov_dealloc_coefs.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_errorreport.interface"
#include "rttov_user_options_checkinput.interface"
#include "rttov_print_opts.interface"
#include "rttov_print_profile.interface"


  !--------------------------
  !
  Integer(Kind=jpim) :: iup=20      ! unit for profile file
  Integer(Kind=jpim) :: ioout=21    ! unit for output

  ! One profile per call and one sensor only for this simple example
  Integer (Kind=jpim) :: nprof = 1           ! Number of profiles per call
  Integer(Kind=jpim)  :: nrttovid = 1        ! Number of sensor coeff files to read

  ! RTTOV_errorhandling interface
  !====================
  Integer(Kind=jpim) :: Err_Unit        ! Logical error unit (<0 for default)
  Integer(Kind=jpim) :: verbosity_level ! (<0 for default)

  ! RTTOV_setup interface
  !====================
  Integer(Kind=jpim)              :: setup_errorstatus  ! setup return code
  Integer(Kind=jpim), Allocatable :: instrument(:,:)    ! platform id, sat id and sensor id
  Type(rttov_options)             :: opts               ! Options structure
  Type(rttov_coefs), Allocatable  :: coefs(:)           ! Coefficients structure
  

  ! RTTOV interface
  !====================
  Integer(Kind=jpim)                :: rttov_errorstatus  ! rttov error return code
  Integer(Kind=jpim)                :: nchannels, nchanprof
  Type(rttov_chanprof), Allocatable :: chanprof(:)
  Type(profile_Type), Allocatable   :: profiles(:)    
  Logical(Kind=jplm), Allocatable   :: calcemis(:)
  Real(Kind=jprb), Allocatable      :: emissivity_in (:)
  Real(Kind=jprb), Allocatable      :: emissivity_out (:)
  Type(transmission_Type)           :: transmission ! transmittances and layer optical depths
  Type(radiance_Type)               :: radiance

  Integer(Kind=jpim) :: alloc_status(20)
  Integer(Kind=jpim) :: errorstatus
  Real(Kind=jprb), Allocatable :: emissivity(:)
  Character (len=80) :: errMessage
  Character (len=11)  :: NameOfRoutine = 'example_fwd'

  ! variables for input
  !====================
  Integer(Kind=jpim), Parameter :: mxchn = 9000 ! max number of channels
  Integer(Kind=jpim) :: input_chan(mxchn)
  Real(Kind=jprb)    :: input_ems(mxchn)
  Real(Kind=jprb)    :: zenith
  Real(Kind=jprb)    :: azimut
  Real(Kind=jprb)    :: lat
  Real(Kind=jprb)    :: zerht
  Real(Kind=jprb)    :: sunzang
  Real(Kind=jprb)    :: sunazang
  Integer(Kind=jpim) :: nlevels
  Integer(Kind=jpim) :: ivch, ich
  Integer(Kind=jpim) :: asw          ! allocate or deallocate switch
  Real(Kind=jprb)    :: ems_val
  Integer(Kind=jpim), Allocatable :: nchan(:) ! number of channels per profile
  Integer(Kind=jpim) :: isurf
  Integer(Kind=jpim) :: nwater
  logical(Kind=jplm) :: addrefrac
  logical(Kind=jplm) :: addsolar
  logical(Kind=jplm) :: addaerosl
  logical(Kind=jplm) :: addclouds
  logical(Kind=jplm) :: all_channels
  Logical(Kind=jplm) :: addinterp  ! switch for the interpolator
  ! printing arrays
  Real(Kind=jprb), Allocatable :: pr_radcld(:)
  Real(Kind=jprb), Allocatable :: pr_trans(:)
  Real(Kind=jprb), Allocatable :: pr_emis(:)
  Real(Kind=jprb), Allocatable :: pr_trans_lev(:,:)
  ! loop variables
  Integer(Kind=jpim) :: j, jch
  Integer(Kind=jpim) :: np, nch
  Integer(Kind=jpim) :: ilev, nprint
  Integer(Kind=jpim) :: iprof, joff
  Integer            :: ios

  !- End of header --------------------------------------------------------
  
  errorstatus     = 0_jpim
  alloc_status(:) = 0_jpim
  allocate (instrument(3,nrttovid),stat= alloc_status(1))

  !=====================================================
  !========== Interactive inputs == start ==============
  Write(0,*) 'enter platform number'
  Read(*,*) instrument(1,nrttovid)
  Write(0,*) 'enter satellite number '
  Read(*,*) instrument(2,nrttovid)
  Write(0,*) 'enter instrument number'
  Read(*,*) instrument(3,nrttovid)
  Write(0,*) 'enter surface type (0=land, 1=sea, 2=ice/snow)'
  Read(*,*) isurf
  Write(0,*) 'enter water type (0=fresh water, 1=ocean water)'
  Read(*,*) nwater
  Write(0,*) 'enter number of profile levels'
  Read(*,*) nlevels
  Write(0,*) 'enter zenith angle in degrees'
  Read(*,*) zenith
  !
  ! Prescribe other inputs
  azimut = 0._jprb   ! Satellite azimuth angle
  sunzang = 0._jprb  ! solar zenith angle
  sunazang = 0._jprb ! solar azimuth angle
  lat = 0._jprb      ! profile latitude 
  zerht = 0._jprb    ! elevation of surface
  !
  ! Set flags
  addrefrac=.True.     ! include refraction in path calc
  addsolar=.False.     ! Do not include reflected solar
  addaerosl=.False.    ! Don't include aerosol effects
  addclouds=.False.    ! Don't include cloud effects
  all_channels=.True.  ! Read all channels into memory from coef file 
  addinterp = .True.   ! Allow interpolation of input profile 
  !
  Allocate (nchan(nprof))
  nchan(:)  = 0_jpim
  nchannels = 0_jpim
  Read(*,*,iostat=ios) ich, ivch, ems_val ! channel number, validity, emissivity
  Do While (ios == 0 )
     If( ivch /= 0 ) Then
        nchannels = nchannels + 1_jpim
        input_chan(nchannels) = ich
        input_ems(nchannels)  = ems_val
     Endif
     Read(*,*,iostat=ios) ich, ivch, ems_val
  End Do
  
  nchan(:) = nchannels
  nchanprof=SUM(nchan(:))  ! here it is also nchannels * nprof
  
  !Pack channels and emmissivity arrays
  Allocate(chanprof(nchanprof))   ! Note these array sizes nchan can vary per profile
  Allocate(emissivity(nchanprof)) ! but for this example assume 1 profile/call with same channels
  Allocate(emissivity_out(nchanprof))
  
  ! Build the list of profile indices
  nch = 0_jpim
  Do j = 1 , nprof
    DO  jch = 1,nchan(j)
      nch = nch + 1_jpim
      chanprof(nch)%prof = j
      chanprof(nch)%chan = input_chan(jch)
      emissivity(nch)    = input_ems(jch)
    End Do
  End Do
  
  ! Initialise options structure
  opts % addrefrac  = addrefrac
  opts % addinterp  = addinterp
  opts % addsolar   = addsolar
  opts % addclouds  = addclouds
  opts % addaerosl  = addaerosl
  opts % ozone_data = .True.       ! we have an ozone profile  
  opts % co2_data   = .False.      ! we do not have profiles
  opts % n2o_data   = .False.      ! for any other constituents
  opts % ch4_data   = .False.      !
  opts % co_data    = .False.      !
  opts % clw_data   = .False.      !
  
  !========== Interactive inputs == end ==============
  !===================================================
  
  !Initialise error management with default value for
  !the error unit number and all error message output
  Err_unit = -1
  verbosity_level = 3_jpim 
  Call rttov_errorhandling(Err_unit, verbosity_level)

  allocate (coefs(nrttovid),stat= alloc_status(1))
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for coefs")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  !Read and initialise coefficients
  Call rttov_setup (&
      & setup_errorstatus,  &! out
      & Err_unit,           &! in
      & verbosity_level,    &! in
      & opts,               &! in
      & coefs(nrttovid),    &! out
      & instrument)          ! in
  if( setup_errorstatus /= errorstatus_success ) then
     write ( *,* ) 'rttov_setup fatal error'
     stop
  endif

  ! security if input number of channels is higher than number
  ! stored in coeffs
  If( nchannels > coefs(nrttovid) % coef % fmv_chn ) Then
      nchannels = coefs(nrttovid) % coef % fmv_chn
      nchan(nprof) = coefs(nrttovid) % coef % fmv_chn
  Endif

  !Ensure the options and coefficients are consistent
  Call rttov_user_options_checkinput(opts, coefs(nrttovid), errorstatus)
  If( errorstatus /= errorstatus_success ) Then
     write ( *,* ) 'error in rttov options'
     Stop
  Endif

  !Open output file which prints results
  Open(IOOUT,file='output_example_fwd.dat',status='unknown',form='formatted',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening the output file ios= ',ios
     Stop
  Endif

  !===============================================
  !========== Read profile == start ==============
  Open(iup, file='prof.dat',status='old',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening profile file ios= ',ios
     Stop
  Endif

  ! Do allocation of input profile arrays with the number of levels.
  asw = 1 ! allocate
  allocate( profiles(nprof),stat= alloc_status(1))
  profiles(1) % nlevels = nlevels
  call rttov_alloc_prof(         &
      & errorstatus,             &
      & nprof,                   &
      & profiles,                &
      & nlevels,                 &
      & opts,                    &
      & asw,                     &
      & coefs = coefs(nrttovid), &
      & init = .true._jplm       )
  If( errorstatus /= errorstatus_success ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for profile arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  ! Read pressure (hPa), temp (K), WV (ppmv), O3 (ppmv)
  ! take care of doing the unit conversions to
  ! hPa, K and ppmv
  Read(iup,*) profiles(1) % p(:)
  Read(iup,*) profiles(1) % t(:)
  Read(iup,*) profiles(1) % q(:)
  Read(iup,*) profiles(1) % o3(:)

  ! 2 meter air variables
  Read(iup,*) profiles(1) % s2m % t ,&
        & profiles(1) % s2m % q ,&
        & profiles(1) % s2m % p ,&
        & profiles(1) % s2m % u ,&
        & profiles(1) % s2m % v 

  ! Skin variables
  Read(iup,*) profiles(1) % skin % t ,&
        & profiles(1) % skin % fastem 

  ! Cloud variables
  Read(iup,*) profiles(1) % ctp,&
        & profiles(1) % cfraction 
  Close(iup)
  
  ! Other variables from interactive inputs
  profiles(1) % zenangle    = zenith
  profiles(1) % azangle     = azimut
  profiles(1) % latitude    = lat
  profiles(1) % elevation   = zerht
  profiles(1) % sunzenangle = sunzang
  profiles(1) % sunazangle  = sunazang
  ! surface type
  profiles(1) % skin % surftype  = isurf
  profiles(1) % skin % watertype = nwater
  profiles(1) % idg         = 0._jprb
  profiles(1) % ish         = 0._jprb
  if( opts % addclouds ) then
    profiles(1) % cloud(:,:) = 0._jprb
    profiles(1) % cfrac(:,:) = 0._jprb
    profiles(1) % icede(:) = 0._jprb
  endif
  !========== Read profile == end ==============
  !=============================================
  ! copy profile 1 to 2..nprof
  If ( nprof > 1_jpim ) Then
    Do iprof = 2, nprof
      Call rttov_copy_prof(      &
            & profiles(iprof:iprof),  &
            & profiles(1_jpim:1_jpim), &
            & larray=.true._jplm,    &
            & lscalar=.true._jplm)
      ! here can introduce some variability
      profiles(iprof) % skin % t = profiles(1) % skin % t + (iprof-1)
    End Do
  End If

  ! allocate radiance results arrays with number of channels
  asw = 1 ! allocate
  call rttov_alloc_rad( &
      & errorstatus,    &
      & nchanprof,      &
      & radiance,       &
      & nlevels-1_jpim,      &
      & asw)
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for radiance arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  
  Allocate( calcemis ( nchanprof ) ,stat= alloc_status(1))
  Allocate( emissivity_in ( nchanprof ) ,stat= alloc_status(2))
  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error in emissivity arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  ! allocate transmittance structure
  call rttov_alloc_transmission( &
      & errorstatus,             &
      & transmission,            &
      & nlevels-1_jpim,          &
      & nchanprof,               &
      & asw,                     &
      & init = .true._jplm)  
  If( errorstatus /= errorstatus_success) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for transmission arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif

  ! save input values of emissivities for all calculations
  ! calculate emissivity where the input emissivity value is less than 0.01
  emissivity_in(:) = emissivity(:)
  calcemis(:) = emissivity(:) < 0.01_JPRB

  ! Call RTTOV forward model
  call rttov_direct(         &
        & rttov_errorstatus, &! out   error flag
        & chanprof,          &! in    channel and profile index structure
        & opts,              &! in    options structure
        & profiles,          &! in    profile array
        & coefs(nrttovid),   &! in    coefficients strucutre
        & calcemis,          &! in    flag for intermal emissivity calc
        & emissivity_in,     &! in    input emissivities per channel
        & emissivity_out,    &! out   emissivities used by RTTOV per channel
        & transmission,      &! inout computed transmittances
        & radiance)           ! inout computed radiances
   
  If ( rttov_errorstatus /= errorstatus_success ) Then
     Write ( 0, * ) 'rttov_direct error'
     Stop
  End If

  ! transfer data to printing arrays
  Allocate(pr_radcld(nchannels), stat= alloc_status(1))
  Allocate(pr_trans(nchannels), stat= alloc_status(2))
  Allocate(pr_emis(nchannels), stat= alloc_status(3))
  Allocate(pr_trans_lev(nlevels,nchannels), stat= alloc_status(4))
  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for printing arrays")' )
     Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  Do iprof = 1, nprof
    pr_radcld(:) = 0.0_JPRB
    pr_trans(:) = 0.0_JPRB
    pr_emis(:) = 0.0_JPRB
    pr_trans_lev(:,:) = 0.0_JPRB
    !
    joff = (iprof-1_jpim) * nchannels
    Do j = 1 , nchannels
      pr_radcld(j) = radiance % cloudy(j+joff)
      pr_trans(j)  = Transmission % tau_total(j+joff)
      pr_emis(j)   = emissivity_out(j+joff )
      Do ilev = 1 ,  nlevels
          pr_trans_lev(ilev,j) = Transmission % tau_levels(ilev,J+joff)
      Enddo
    Enddo
    !
    !     OUTPUT RESULTS
    !
    NPRINT = 1+ Int((nchannels-1)/10)
    Write(IOOUT,*)' -----------------'
    Write(IOOUT,*)' Instrument ', inst_name(instrument(3,nrttovid))
    Write(IOOUT,*)' -----------------'
    Write(IOOUT,*)' '
    Write(IOOUT,*)' Profile ',iprof
    Write(IOOUT,*)' '

    Call rttov_print_opts(opts, lu=IOOUT)
    Call rttov_print_profile(profiles(iprof), lu=IOOUT)
    WRITE(IOOUT,777) platform_name(instrument(1,nrttovid)), instrument(2,nrttovid)
  
    WRITE(IOOUT,*)'CHANNELS PROCESSED:'
    WRITE(IOOUT,111) (chanprof(j) % chan, j = 1+joff,nchannels+joff)
    WRITE (IOOUT,*)' '
    Write(IOOUT,222) (radiance % bt(j), j = 1+joff,nchannels+joff)
    Write(IOOUT,*)' '
    Write(IOOUT,*)'CALCULATED RADIANCES: SAT =', instrument(2,nrttovid)
    Write(IOOUT,222) (radiance % total(j), j = 1+joff,nchannels+joff)
    Write(IOOUT,*)' '
    Write(IOOUT,*)'CALCULATED OVERCAST RADIANCES: SAT =', instrument(2,nrttovid)
    Write(IOOUT,222) (pr_radcld(j), j = 1,nchannels)
    Write (IOOUT,*)' '
    Write(IOOUT,*)'CALCULATED SURFACE TO SPACE TRANSMITTANCE: S'&
              & ,'AT =',instrument(2,nrttovid) 
    Write(IOOUT,4444) (pr_trans(j), j = 1,nchannels)
    Write (IOOUT,*)' '
    Write(IOOUT,*)'CALCULATED SURFACE EMISSIVITIES '&
              & ,'SAT =',instrument(2,nrttovid) 
    Write(IOOUT,444) (pr_emis(j), j = 1,nchannels)
    !
    !
    If(nchan(nprof) <= 20)Then
      Do  NP = 1 , NPRINT
          Write (IOOUT,*)' '
          Write (IOOUT,*)'Level to space transmittances for channels'
          Write(IOOUT,1115) (chanprof(j+joff) % chan,&
                  & J = 1+joff+(NP-1)*10,Min(Int(10+joff+(NP-1)*10,jpim),nchannels)) 
          Do  ILEV = 1 , NLEVELS
            Write(IOOUT,4445)ILEV,(pr_trans_lev(ilev,J),&
                    & J = 1+(NP-1)*10,Min(Int(10+(NP-1)*10,jpim),nchannels)) 
          End Do
          Write(IOOUT,1115) (chanprof(j+ joff) % chan,&
                  & J = 1+joff+(NP-1)*10,Min(Int(10+joff+(NP-1)*10,jpim),nchannels)) 
      End Do
    Endif
  End Do
  !
  ! Deallocate arrays
  deallocate( chanprof,       stat=alloc_status(1))
  deallocate( emissivity,     stat=alloc_status(2))
  deallocate( emissivity_in,  stat=alloc_status(3))
  deallocate( emissivity_out, stat=alloc_status(4))
  deallocate( calcemis,       stat=alloc_status(5))

  ! dealloc printing arrays
  deallocate( pr_radcld,      stat=alloc_status(6))
  deallocate( pr_trans,       stat=alloc_status(7))
  deallocate( pr_emis,        stat=alloc_status(8))
  deallocate( pr_trans_lev,   stat=alloc_status(9))
  If( any(alloc_status /= 0) ) then
    errorstatus = errorstatus_fatal
    Write( errMessage, '( "mem deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
    Stop
  End If

  asw = 0 ! deallocate radiance arrays
  call rttov_alloc_rad (errorstatus,nchannels,radiance,nlevels-1_jpim,asw)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "radiance deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif

  asw = 0 ! deallocate transmission arrays
  call rttov_alloc_transmission (errorstatus,transmission,nlevels-1_jpim,nchannels,asw)
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "radiance deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif
  
  asw = 0 ! deallocate profile arrays
  call rttov_alloc_prof (errorstatus,nprof,profiles,nlevels,opts,asw)
  deallocate(profiles,stat=alloc_status(1))
  If(errorstatus /= errorstatus_success .or. alloc_status(1) /= 0) Then
    Write( errMessage, '( "profile deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif

  Call rttov_dealloc_coefs (errorstatus, coefs(nrttovid))
  If(errorstatus /= errorstatus_success) Then
    Write( errMessage, '( "coefs deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif
  Deallocate(coefs,stat=alloc_status(1))
  If(errorstatus /= errorstatus_success .or. alloc_status(1) /= 0) Then
    Write( errMessage, '( "coefficients array deallocation error")' )
    Call rttov_errorreport (errorstatus, errMessage, NameOfRoutine)
  Endif
   
  !Close output file
  Close(IOOUT,iostat=ios)
  If( ios /= 0 ) Then
    Write(*,*) 'error closing the output file ios= ',ios
    Stop
  Endif

111  FORMAT(1X,10I8)
1115 Format(3X,10I8)
222  Format(1X,10F8.2)
444  Format(1X,10F8.3)
4444 Format(1X,10F8.4)
4445 Format(1X,I2,10F8.4)
777  FORMAT(/,'CALCULATED BRIGHTNESS TEMPERATURES: SAT = ',A8,I3 )

End Program example_fwd
