Program rttov_test

  !
  !
  ! Copyright:
  !
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          06/2008    Creation P. Marguinaud & P. Brunel
  !
  ! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
  !

  Use rttov_const, Only :      &
       & errorstatus_success,  &
       & errorstatus_warning,  &
       & errorstatus_fatal,    &
       & sensor_id_mw,         &
       & sensor_id_po,         &
       & sensor_id_ir,         &
       & sensor_id_hi,         &
       & surftype_sea

  Use rttov_types, Only :     &
       & rttov_coefs,         &
       & rttov_options,       &
       & rttov_pccomp,        &
       & profile_Type,        &
       & transmission_Type,   &
       & radiance_Type,       &
       & rttov_chanprof,      &
       & sskin_type,          &
       & s2m_type,            &
       & rttov_traj

#include "throw.h"

  Use parkind1, Only : jpim, jprb, jplm

  Use rttov_getoptions, Only : getoption
  Use yomhook, Only : lhook, dr_hook
  Use rttov_lun
  Use rttov_unix_env, Only: rttov_exit
  
  Implicit None



#include "rttov_direct.interface"
#include "rttov_tl.interface"
#include "rttov_ad.interface"
#include "rttov_k.interface"
#include "rttov_k_tl.interface"
#include "rttov_k_ad.interface"
#include "rttov_k_bf.interface"


#include "rttov_parallel_direct.interface"
#include "rttov_parallel_tl.interface"
#include "rttov_parallel_ad.interface"
#include "rttov_parallel_k.interface"

#include "rttov_make_profile_inc.interface"
#include "rttov_scale_profile_inc.interface"
#include "rttov_make_radiance_inc.interface"
#include "rttov_scale_radiance_inc.interface"
#include "rttov_make_pccomp_inc.interface"
#include "rttov_scale_pccomp_inc.interface"

#include "rttov_init_coefs.interface"
#include "rttov_read_coefs.interface"
#include "rttov_dealloc_coefs.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_pccomp.interface"
#include "rttov_init_prof.interface"
#include "rttov_add_prof.interface"
#include "rttov_copy_prof.interface"
#include "rttov_copy_rad.interface"
#include "rttov_copy_pccomp.interface"
#include "rttov_alloc_traj.interface"

#include "rttov_errorreport.interface"


  Character*(*), Parameter :: Gformat = 'G15.6E3'


! Test definition; we have here all dimensions, flags, etc...
  Type rttov_test_defn
! dimensions
    Integer(Kind=jpim) :: nlevels = -1_jpim, nchannels = -1_jpim, nprofiles = -1_jpim, mult = 1_jpim
    Integer(Kind=jpim) :: nchannels_rec = -1_jpim, npcscores = -1_jpim

! coefficients files
    Character*256      :: f_coef = "", f_scat = "", f_optp = "", f_pccomp = ""
    Character*256      :: coef_format = 'formatted'            
    Character*256      :: coef_prefix

! flags

    Type(rttov_options) :: opts

! input files
    Character*256      :: f_p    = "", f_t   = "", f_q      = "", f_o3      = "", f_co2    = "", f_co    = "",  &
                          f_ch4  = "", f_clw = "", f_aerosl = "", f_cloud   = "", f_cfrac  = "", f_n2o   = "",  &
                          f_skin = "", f_s2m = "", f_angles = "", f_aerosli = "", f_cloud0 = "", f_flags = "",  &
                          f_icede = "", f_be   = "", f_elevation = ""
    Character*256      :: f_channels = "", f_emissivity = "", f_calcemis = "", f_lprofiles = "", &
                          f_pcscores = "", f_channels_rec = ""

! what we will do
   Logical(Kind=jplm) :: do_direct = .FALSE., do_tl   = .FALSE., do_ad   = .FALSE., do_k = .FALSE.,            &
                         do_k_bf   = .FALSE., do_k_tl = .FALSE., do_k_ad = .FALSE., do_taylor = .FALSE.
   Logical(Kind=jplm) :: do_print  = .FALSE., printallrad = .FALSE., ltemp = .FALSE., lallchans = .FALSE.
   Integer(Kind=jpim) :: ntimes = 1, nthreads = 0     
   Logical(Kind=jplm) :: prof_by_prof = .FALSE. ! run RTTOV profile by profile
   Logical(Kind=jplm) :: chan_by_chan = .FALSE. ! run RTTOV channel by channel
   Real(Kind=jprb)    :: scale_inc = 1.0_jprb
   Real(Kind=jprb)    :: scale_out = 1.0_jprb
  End Type

! The basic data structures passed to rttov_direct/rttov_tl/rttov_ad/rttov_k
  Type rttov_input_data
   Type(profile_type), Pointer :: profiles(:)         => NULL()
   Type(profile_type), Pointer :: profiles_pert(:)    => NULL()
   Real(Kind=jprb),    Pointer :: emissivity(:)       => NULL()
   Real(Kind=jprb),    Pointer :: emissivity_pert(:)  => NULL()
   Real(Kind=jprb),    Pointer :: emissivity_out(:)   => NULL()
   Type(transmission_Type)     :: transmission
   Type(radiance_Type)         :: radiance
   Type(radiance_Type)         :: radiance_saved
   Type(rttov_pccomp)          :: pccomp
   Type(rttov_pccomp)          :: pccomp_saved
  End Type
 
  Type rttov_k_input_data
   Type(profile_type),   Pointer :: profiles_k_pc(:) => NULL()
   Type(profile_type),   Pointer :: profiles_k_rec(:) => NULL()
  End Type

! Test data. All dynamically allocated data goes here.
  Type rttov_test_data
   Type(rttov_coefs)             :: coefs
   Type(rttov_chanprof), Pointer :: chanprof(:)          => NULL()
   Integer(Kind=jpim)            :: rttov_errorstatus
   Type(rttov_input_data)        :: direct, tl, ad, k, k_bf, k_tl, k_ad
   Type(rttov_k_input_data)      :: data_k, data_k_bf, data_k_tl, data_k_ad
   Logical(Kind=jplm),   Pointer :: calcemis(:)          => NULL()
   Type(rttov_traj)              :: traj, traj_tl, traj_ad, traj_k
   Integer(Kind=jpim),   Pointer :: channels_rec(:)      => NULL()
  End Type

  Type rttov_test_struct
    Integer(Kind=jpim)    :: rank
    Character*256         :: path
    Type(rttov_test_defn) :: defn
    Type(rttov_test_data) :: data
    Integer(Kind=jpim)    :: err = 0_jpim
  End Type


  Type(rttov_test_struct), Pointer :: ts(:)     => NULL()
  Character*256,           Pointer :: paths(:)  => NULL()
  Integer(Kind=jpim) :: ip, np
  Real(Kind=jprb)    :: zhook_handle
  Integer(Kind=jpim) :: nthreads_multi_run
  Integer(Kind=jpim) :: err

TRY

  If(lhook) Call dr_hook('rttov_test',0_jpim,zhook_handle)

  Call GetOption( "--test-path-list", paths )

  nthreads_multi_run = 1_jpim
  Call GetOption( "--nthreads-multi-run", nthreads_multi_run )

  If( .not. associated( paths ) ) Then
    Allocate( paths(1), stat = err )
    THROWM(err.ne.0,"Allocation of paths failed")
    paths(1) = "."
  EndIf

  np = Size(paths)

  Allocate( ts( np ), stat = err )
  THROWM(err.ne.0,"Allocation of ts failed")
  Do ip = 1, np
    ts%rank = ip
  EndDo


  Do ip = 1, np
    call setup( ts(ip), paths(ip), err )
    THROW(err.ne.0)
  EndDo
  Deallocate( paths, stat = err )
  THROWM(err.ne.0,"DeAllocation of paths failed")

  Do ip = 1, np

    if(err.ne.0)cycle

    If( ts(ip)%defn%ltemp ) Then
      call run(                                   &
        ts(ip),                                   &
        ts(ip)%err,                               &
        ts(ip)%data%traj,    ts(ip)%data%traj_tl, &
        ts(ip)%data%traj_ad, ts(ip)%data%traj_k )
    Else
      call run( ts(ip), ts(ip)%err )
    EndIf

    THROW_L(ts(ip)%err.ne.0,800)

    cycle

    CATCH_L(800)

    if( ts(ip)%err .ne. 0 ) then
!$OMP FLUSH(err)
      err = errorstatus_fatal
    endif

  EndDo
  THROW(err.ne.0)

  Do ip = 1, np
    call cleanup( ts(ip), err )  
    THROW(err.ne.0)
  EndDo

  Deallocate( ts, stat = err )
  THROWM(err.ne.0,"Deallocation of ts")

  If(lhook) Call dr_hook('rttov_test',1_jpim,zhook_handle)

PCATCH
  If(lhook) Call dr_hook('rttov_test',1_jpim,zhook_handle)

  Call rttov_exit(1_jpim)

  
  Contains

  Subroutine run( ts, err, traj, traj_tl, traj_ad, traj_k )
!
  Type(rttov_test_struct),    Target, Intent(inout) :: ts
  Integer(Kind=jpim),                 Intent(out)   :: err
  Type(rttov_traj), Optional, Target, Intent(inout) :: traj, traj_tl, traj_ad, traj_k
!
  Integer(Kind=jpim), Parameter :: niter = 10
  Integer(Kind=jpim)            :: itime, iter, i, j
  Integer(Kind=jpim)            :: nchanrec, npcscores
  Real(Kind=jprb), Allocatable  :: sumdir(:,:), sumtl(:,:)
  
!
TRY

  Do itime = 1, ts%defn%ntimes

!    verbose_checkinput_warnings = ts%verbose_checkinput_warnings
  
    If( ts%defn%do_direct ) Then

      If( (ts%defn%nthreads .le. 0) .and. ts%defn%prof_by_prof ) Then

        call rttov_parallel_direct(                                         &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,              &
          & ts%defn%opts,                    ts%data%direct%profiles,       &
          & ts%data%coefs,                   ts%data%calcemis,              &
          & ts%data%direct%emissivity,       ts%data%direct%emissivity_out, &
          & ts%data%direct%transmission,     ts%data%direct%radiance,       &
          & nthreads = 1_jpim,               strategy = 1_jpim )

      Else If( (ts%defn%nthreads .le. 0) .and. ts%defn%chan_by_chan ) Then

        call rttov_parallel_direct(                                         &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,              &
          & ts%defn%opts,                    ts%data%direct%profiles,       &
          & ts%data%coefs,                   ts%data%calcemis,              &
          & ts%data%direct%emissivity,       ts%data%direct%emissivity_out, &
          & ts%data%direct%transmission,     ts%data%direct%radiance,       &
          & nthreads = 1_jpim,               strategy = 2_jpim )
      
      Else If( ts%defn%nthreads .le. 0 ) Then
      
        call rttov_direct(                       &
          & ts%data%rttov_errorstatus,           &
          & ts%data%chanprof,                    &
          & ts%defn%opts,                        &
          & ts%data%direct%profiles,             &
          & ts%data%coefs,                       &
          & ts%data%calcemis,                    &
          & ts%data%direct%emissivity,           &
          & ts%data%direct%emissivity_out,       &
          & ts%data%direct%transmission,         &
          & ts%data%direct%radiance,             &
          & pccomp       = ts%data%direct%pccomp,&
          & channels_rec = ts%data%channels_rec, &
          & traj         = traj )


      Else

        call rttov_parallel_direct(                                         &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,              &
          & ts%defn%opts,                    ts%data%direct%profiles,       &
          & ts%data%coefs,                   ts%data%calcemis,              &
          & ts%data%direct%emissivity,       ts%data%direct%emissivity_out, &
          & ts%data%direct%transmission,     ts%data%direct%radiance,       &
          & pccomp       = ts%data%direct%pccomp,                           &
          & channels_rec = ts%data%channels_rec,                            &
          & nthreads     = ts%defn%nthreads)

      EndIf

      If( ts%data%rttov_errorstatus /= 0 ) Then
        err = errorstatus_fatal
        THROW(err.ne.0)
      EndIf
      
      If( ts%defn%do_taylor ) Then
        If( ts%defn%opts%addpc ) Then
          Call rttov_copy_pccomp(ts%data%direct%pccomp_saved, ts%data%direct%pccomp)
        Else
          Call rttov_copy_rad(ts%data%direct%radiance_saved, ts%data%direct%radiance)
        EndIf
      EndIf
      
      If( ts%defn%do_print ) Then
        Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE", Trim(ts%path)//"/direct/radiance.txt", ts%defn%printallrad )
        THROW(err.ne.0)
        Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION", Trim(ts%path)//"/direct/transmission.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/direct/emissivity_out.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addpc ) Then
          Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/direct/pcscores.txt" )
          THROW(err.ne.0)
        EndIf
      EndIf
      
      ts%defn%opts%verbose_checkinput_warnings = .false.
    
    EndIf 
    
    If( ts%defn%do_tl ) Then
  
      If( (ts%defn%nthreads .le. 0) .and. ts%defn%prof_by_prof ) Then

        call rttov_parallel_tl(                                             &
          & ts%data%rttov_errorstatus,      ts%data%chanprof,               &
          & ts%defn%opts,                   ts%data%direct%profiles,        &
          & ts%data%tl%profiles,            ts%data%coefs,                  &
          & ts%data%calcemis,               ts%data%direct%emissivity,      &
          & ts%data%tl%emissivity,          ts%data%direct%emissivity_out,  &
          & ts%data%tl%emissivity_out,      ts%data%direct%transmission,    &
          & ts%data%tl%transmission,        ts%data%direct%radiance,        &
          & ts%data%tl%radiance,            nthreads = 1_jpim,              & 
          & strategy = 1_jpim )

      Else If( (ts%defn%nthreads .le. 0) .and. ts%defn%chan_by_chan ) Then

        call rttov_parallel_tl(                                             &
          & ts%data%rttov_errorstatus,      ts%data%chanprof,               &
          & ts%defn%opts,                   ts%data%direct%profiles,        &
          & ts%data%tl%profiles,            ts%data%coefs,                  &
          & ts%data%calcemis,               ts%data%direct%emissivity,      &
          & ts%data%tl%emissivity,          ts%data%direct%emissivity_out,  &
          & ts%data%tl%emissivity_out,      ts%data%direct%transmission,    &
          & ts%data%tl%transmission,        ts%data%direct%radiance,        &
          & ts%data%tl%radiance,            nthreads = 1_jpim,              & 
          & strategy = 2_jpim )
          
      Else If( ts%defn%nthreads .le. 0 ) Then
    
        call rttov_tl(                             &
          & ts%data%rttov_errorstatus,             &
          & ts%data%chanprof,                      &
          & ts%defn%opts,                          &
          & ts%data%direct%profiles,               &
          & ts%data%tl%profiles,                   &
          & ts%data%coefs,                         &
          & ts%data%calcemis,                      &
          & ts%data%direct%emissivity,             &
          & ts%data%tl%emissivity,                 &
          & ts%data%direct%emissivity_out,         &
          & ts%data%tl%emissivity_out,             &
          & ts%data%direct%transmission,           &
          & ts%data%tl%transmission,               &
          & ts%data%direct%radiance,               &
          & ts%data%tl%radiance,                   &
          & channels_rec = ts%data%channels_rec,   &
          & pccomp       = ts%data%direct%pccomp,  &
          & pccomp_tl    = ts%data%tl%pccomp,      &
          & traj         = traj,                   &
          & traj_tl      = traj_tl )

      Else

        call rttov_parallel_tl(                                             &
          & ts%data%rttov_errorstatus,      ts%data%chanprof,               &
          & ts%defn%opts,                   ts%data%direct%profiles,        &
          & ts%data%tl%profiles,            ts%data%coefs,                  &
          & ts%data%calcemis,               ts%data%direct%emissivity,      &
          & ts%data%tl%emissivity,          ts%data%direct%emissivity_out,  &
          & ts%data%tl%emissivity_out,      ts%data%direct%transmission,    &
          & ts%data%tl%transmission,        ts%data%direct%radiance,        &
          & ts%data%tl%radiance,                                            &
          & channels_rec = ts%data%channels_rec,                            &
          & pccomp       = ts%data%direct%pccomp,                           &
          & pccomp_tl    = ts%data%tl%pccomp,                               &
          & nthreads     = ts%defn%nthreads )

      EndIf
        
      If( ts%data%rttov_errorstatus /= 0 ) Then
        err = errorstatus_fatal
        THROW(err.ne.0)
      EndIf
        
      If( ts%defn%do_print ) Then
        If( ts%defn%scale_out /= 1.0_jprb ) Then
          ts%data%tl%radiance%total = ts%data%tl%radiance%total * ts%defn%scale_out
          ts%data%tl%radiance%bt    = ts%data%tl%radiance%bt * ts%defn%scale_out
          ts%data%tl%radiance%clear = ts%data%tl%radiance%clear * ts%defn%scale_out
          ts%data%tl%radiance%bt_clear = ts%data%tl%radiance%bt_clear * ts%defn%scale_out
          ts%data%tl%radiance%overcast = ts%data%tl%radiance%overcast * ts%defn%scale_out
          ts%data%tl%transmission%tau_total = ts%data%tl%transmission%tau_total * ts%defn%scale_out
          ts%data%tl%transmission%tau_levels = ts%data%tl%transmission%tau_levels * ts%defn%scale_out
          ts%data%tl%emissivity_out = ts%data%tl%emissivity_out * ts%defn%scale_out
          If( ts%defn%opts%addpc ) Then
            If(Associated(ts%data%tl%pccomp%pcscores)) &
             & ts%data%tl%pccomp%pcscores = ts%data%tl%pccomp%pcscores * ts%defn%scale_out
            If(Associated(ts%data%tl%pccomp%clear_pccomp)) &
             & ts%data%tl%pccomp%pcscores = ts%data%tl%pccomp%clear_pccomp * ts%defn%scale_out
            If(Associated(ts%data%tl%pccomp%bt_pccomp)) &
             & ts%data%tl%pccomp%pcscores = ts%data%tl%pccomp%bt_pccomp * ts%defn%scale_out          
          End If
        End If

        Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE", Trim(ts%path)//"/tl/radiance.txt", ts%defn%printallrad )
        THROW(err.ne.0)
        Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION", Trim(ts%path)//"/tl/transmission.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/tl/emissivity_out.txt" )
        THROW(err.ne.0)
        Call PrintRadiance (err, ts%data%tl%radiance, "RADIANCE_TL", Trim(ts%path)//"/tl/radiance_tl.txt", ts%defn%printallrad )
        THROW(err.ne.0)
        Call PrintTransmission (err, ts%data%tl%transmission, "TRANSMISSION_TL", Trim(ts%path)//"/tl/transmission_tl.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%tl%emissivity_out, "EMISSIVITY_OUT_TL", Trim(ts%path)//"/tl/emissivity_out_tl.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addpc ) Then
          Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/tl/pcscores.txt" )
          THROW(err.ne.0)
          Call PrintPCScores (err, ts%data%tl%pccomp, "PCSCORES_TL", Trim(ts%path)//"/tl/pcscores_tl.txt" )
          THROW(err.ne.0)
        EndIf
      EndIf
        
      ts%defn%opts%verbose_checkinput_warnings = .false.
    
    EndIf 
    
    If( ts%defn%do_ad ) Then
      
      ! Need to initialise the adjoint variables before each call
      call rttov_init_prof(ts%data%ad%profiles)
      ts%data%ad%emissivity(:) = 0._jprb
      ts%data%ad%emissivity_out(:) = 0._jprb
      If( ts%defn%opts%addpc )Then
        call rttov_copy_pccomp(ts%data%ad%pccomp, ts%data%ad%pccomp_saved)
        call rttov_init_rad(ts%data%ad%radiance)
      Else
        call rttov_copy_rad(ts%data%ad%radiance, ts%data%ad%radiance_saved)
      EndIf
      
      If( (ts%defn%nthreads .le. 0) .and. ts%defn%prof_by_prof ) Then

        call rttov_parallel_ad(                                               &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,                &
          & ts%defn%opts,                    ts%data%direct%profiles,         &
          & ts%data%ad%profiles,             ts%data%coefs,                   &
          & ts%data%calcemis,                ts%data%direct%emissivity,       &
          & ts%data%ad%emissivity,           ts%data%direct%emissivity_out,   &
          & ts%data%ad%emissivity_out,       ts%data%direct%transmission,     &
          & ts%data%ad%transmission,         ts%data%direct%radiance,         &
          & ts%data%ad%radiance,             nthreads = 1_jpim,               &
          & strategy = 1_jpim )

      Else If( (ts%defn%nthreads .le. 0) .and. ts%defn%chan_by_chan ) Then

        call rttov_parallel_ad(                                               &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,                &
          & ts%defn%opts,                    ts%data%direct%profiles,         &
          & ts%data%ad%profiles,             ts%data%coefs,                   &
          & ts%data%calcemis,                ts%data%direct%emissivity,       &
          & ts%data%ad%emissivity,           ts%data%direct%emissivity_out,   &
          & ts%data%ad%emissivity_out,       ts%data%direct%transmission,     &
          & ts%data%ad%transmission,         ts%data%direct%radiance,         &
          & ts%data%ad%radiance,             nthreads = 1_jpim,               &
          & strategy = 2_jpim )
          
      Else If( ts%defn%nthreads .le. 0 ) Then
    
        call rttov_ad(                      &
          & ts%data%rttov_errorstatus,      &
          & ts%data%chanprof,               &
          & ts%defn%opts,                   &
          & ts%data%direct%profiles,        &
          & ts%data%ad%profiles,            &
          & ts%data%coefs,                  &
          & ts%data%calcemis,               &
          & ts%data%direct%emissivity,      &
          & ts%data%ad%emissivity,          &
          & ts%data%direct%emissivity_out,  &
          & ts%data%ad%emissivity_out,      &
          & ts%data%direct%transmission,    &
          & ts%data%ad%transmission,        &
          & ts%data%direct%radiance,        &
          & ts%data%ad%radiance,            &
          & channels_rec = ts%data%channels_rec,   &
          & pccomp       = ts%data%direct%pccomp,  &
          & pccomp_ad    = ts%data%ad%pccomp,      &
          & traj         = traj,                   &
          & traj_ad      = traj_ad )

      Else

        call rttov_parallel_ad(                                               &
          & ts%data%rttov_errorstatus,       ts%data%chanprof,                &
          & ts%defn%opts,                    ts%data%direct%profiles,         &
          & ts%data%ad%profiles,             ts%data%coefs,                   &
          & ts%data%calcemis,                ts%data%direct%emissivity,       &
          & ts%data%ad%emissivity,           ts%data%direct%emissivity_out,   &
          & ts%data%ad%emissivity_out,       ts%data%direct%transmission,     &
          & ts%data%ad%transmission,         ts%data%direct%radiance,         &
          & ts%data%ad%radiance,                                              &
          & channels_rec = ts%data%channels_rec,                              &
          & pccomp       = ts%data%direct%pccomp,                             &
          & pccomp_ad    = ts%data%ad%pccomp,                                 &
          & nthreads     = ts%defn%nthreads )
        
      EndIf
      
      If( ts%data%rttov_errorstatus /= 0 ) Then
        err = errorstatus_fatal
        THROW(err.ne.0)
      EndIf
        
      If( ts%defn%do_print ) Then

        If( ts%defn%scale_out /= 1.0_jprb )Then
          Call rttov_scale_profile_inc(ts%data%ad%profiles, ts%defn%scale_out)
        End If

        Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE", Trim(ts%path)//"/ad/radiance.txt", ts%defn%printallrad )
        THROW(err.ne.0)
        Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION", Trim(ts%path)//"/ad/transmission.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/ad/emissivity_out.txt" )
        THROW(err.ne.0)
        Call PrintProfiles ( err, ts%data%ad%profiles, "PROFILES_AD", Trim(ts%path)//"/ad/profiles_ad.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%ad%emissivity * ts%defn%scale_out, "EMISSIVITY_AD", &
          Trim(ts%path)//"/ad/emissivity_ad.txt")
        THROW(err.ne.0)
        If( ts%defn%opts%addpc ) Then
          Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/ad/pcscores.txt" )
          THROW(err.ne.0)
          Call PrintPCScores (err, ts%data%ad%pccomp, "PCSCORES_AD", Trim(ts%path)//"/ad/pcscores_ad.txt" )
          THROW(err.ne.0)
        EndIf
      EndIf
      
      ts%defn%opts%verbose_checkinput_warnings = .false.
    
    EndIf 
    
    If( ts%defn%do_k ) Then
    
      call rttov_init_prof(ts%data%k%profiles)
      ts%data%k%emissivity(:) = 0._jprb
      ts%data%k%emissivity_out(:) = 0._jprb
      
      call rttov_init_rad(ts%data%k%radiance)
      If( ts%defn%opts%switchrad ) Then
        ts%data%k%radiance%bt = 1._jprb
      Else
        ts%data%k%radiance%total = 1._jprb
      EndIf
    
      If( (ts%defn%nthreads .le. 0) .and. ts%defn%prof_by_prof ) Then

        Call rttov_parallel_k(                                           &
          & ts%data%rttov_errorstatus,     ts%data%chanprof,             &
          & ts%defn%opts,                  ts%data%direct%profiles,      & 
          & ts%data%k%profiles,            ts%data%coefs,                & 
          & ts%data%calcemis,              ts%data%direct%emissivity,    & 
          & ts%data%k%emissivity,          ts%data%direct%emissivity_out,& 
          & ts%data%k%emissivity_out,      ts%data%direct%transmission,  & 
          & ts%data%k%transmission,        ts%data%direct%radiance,      & 
          & ts%data%k%radiance,            nthreads = 1_jpim,            &
          & strategy = 1_jpim )

      Else If( (ts%defn%nthreads .le. 0) .and. ts%defn%chan_by_chan ) Then

        Call rttov_parallel_k(                                           &
          & ts%data%rttov_errorstatus,     ts%data%chanprof,             &
          & ts%defn%opts,                  ts%data%direct%profiles,      &
          & ts%data%k%profiles,            ts%data%coefs,                &
          & ts%data%calcemis,              ts%data%direct%emissivity,    &
          & ts%data%k%emissivity,          ts%data%direct%emissivity_out,&
          & ts%data%k%emissivity_out,      ts%data%direct%transmission,  &
          & ts%data%k%transmission,        ts%data%direct%radiance,      &
          & ts%data%k%radiance,            nthreads = 1_jpim,            &
          & strategy = 2_jpim )
          
      Else If( ts%defn%nthreads .le. 0 ) Then
    
        Call rttov_k(                                       &
          & ts%data%rttov_errorstatus,                      &
          & ts%data%chanprof,                               &
          & ts%defn%opts,                                   &
          & ts%data%direct%profiles,                        &
          & ts%data%k%profiles,                             &
          & ts%data%coefs,                                  &
          & ts%data%calcemis,                               &
          & ts%data%direct%emissivity,                      &
          & ts%data%k%emissivity,                           &
          & ts%data%direct%emissivity_out,                  &
          & ts%data%k%emissivity_out,                       &
          & ts%data%direct%transmission,                    &
          & ts%data%k%transmission,                         &
          & ts%data%direct%radiance,                        &
          & ts%data%k%radiance,                             &
          & traj           = traj,                          &
          & traj_k         = traj_k,                        &
          & pccomp         = ts%data%direct%pccomp,         &
          & pccomp_k       = ts%data%k%pccomp,              &
          & profiles_k_rec = ts%data%data_k%profiles_k_rec, &
          & profiles_k_pc  = ts%data%data_k%profiles_k_pc,  &
          & channels_rec   = ts%data%channels_rec )

      Else

        Call rttov_parallel_k(                                           &
          & ts%data%rttov_errorstatus,     ts%data%chanprof,             &
          & ts%defn%opts,                  ts%data%direct%profiles,      &
          & ts%data%k%profiles,            ts%data%coefs,                &
          & ts%data%calcemis,              ts%data%direct%emissivity,    &
          & ts%data%k%emissivity,          ts%data%direct%emissivity_out,&
          & ts%data%k%emissivity_out,      ts%data%direct%transmission,  &
          & ts%data%k%transmission,        ts%data%direct%radiance,      &
          & ts%data%k%radiance,                                          &
          & pccomp         = ts%data%direct%pccomp,                      &
          & pccomp_k       = ts%data%k%pccomp,                           &
          & profiles_k_rec = ts%data%data_k%profiles_k_rec,              &
          & profiles_k_pc  = ts%data%data_k%profiles_k_pc,               &
          & channels_rec   = ts%data%channels_rec,                       &
          & nthreads       = ts%defn%nthreads)
      
      EndIf
    
      If( ts%data%rttov_errorstatus /= 0 ) Then
        err = errorstatus_fatal
        THROW(err.ne.0)
      EndIf
        
      If( ts%defn%do_print ) Then
        Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE",  Trim(ts%path)//"/k/radiance.txt", ts%defn%printallrad )
        THROW(err.ne.0)
        Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION",  Trim(ts%path)//"/k/transmission.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/k/emissivity_out.txt" )
        THROW(err.ne.0)
        Call PrintProfiles ( err, ts%data%k%profiles, "PROFILES_K",  Trim(ts%path)//"/k/profiles_k.txt" )
        THROW(err.ne.0)
        Call PrintEmissivity (err, ts%data%k%emissivity, "EMISSIVITY_K",  Trim(ts%path)//"/k/emissivity_k.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addpc ) Then
          Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/k/pcscores.txt" )
          THROW(err.ne.0)
          Call PrintPCScores (err, ts%data%k%pccomp, "PCSCORES_K", Trim(ts%path)//"/k/pcscores_k.txt" )
          THROW(err.ne.0)
          If( ts%defn%opts%addradrec ) then
            Call PrintProfiles ( err, ts%data%data_k%profiles_k_rec, "PROFILES_K_REC",  Trim(ts%path)//"/k/profiles_k_rec.txt" )
            THROW(err.ne.0)
          else
            Call PrintProfiles ( err, ts%data%data_k%profiles_k_pc, "PROFILES_K_PC",  Trim(ts%path)//"/k/profiles_k_pc.txt" )
            THROW(err.ne.0)
          endif
        EndIf
      EndIf
  
      ts%defn%opts%verbose_checkinput_warnings = .false.
  
    EndIf

  EndDo
    
  If( ts%defn%do_k_bf ) Then
  
    Call rttov_k_bf(                                       &
      & ts%data%rttov_errorstatus,                         &
      & ts%data%chanprof,                                  &
      & ts%defn%opts,                                      &
      & ts%data%direct%profiles,                           &
      & ts%data%k_bf%profiles,                             &
      & ts%data%coefs,                                     &
      & ts%data%calcemis,                                  &
      & ts%data%direct%emissivity,                         &
      & ts%data%k_bf%emissivity,                           &
      & ts%data%direct%emissivity_out,                     &
      & ts%data%direct%transmission,                       &
      & ts%data%direct%radiance,                           &
      & pccomp         = ts%data%direct%pccomp,            &
      & profiles_k_rec = ts%data%data_k_bf%profiles_k_rec, &
      & profiles_k_pc  = ts%data%data_k_bf%profiles_k_pc,  &
      & channels_rec   = ts%data%channels_rec )

    If( ts%data%rttov_errorstatus /= 0 ) Then
      err = errorstatus_fatal
      THROW(err.ne.0)
    EndIf
      
    If( ts%defn%do_print ) Then
      Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE",  Trim(ts%path)//"/k_bf/radiance.txt", ts%defn%printallrad )
      THROW(err.ne.0)
      Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION",  Trim(ts%path)//"/k_bf/transmission.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/k_bf/emissivity_out.txt" )
      THROW(err.ne.0)
      Call PrintProfiles ( err, ts%data%k_bf%profiles, "PROFILES_K",  Trim(ts%path)//"/k_bf/profiles_k.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%k_bf%emissivity, "EMISSIVITY_K",  Trim(ts%path)//"/k_bf/emissivity_k.txt" )
      THROW(err.ne.0)
      If( ts%defn%opts%addpc ) Then
        Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/k_bf/pcscores.txt" )
        THROW(err.ne.0)
        Call PrintPCScores (err, ts%data%k_bf%pccomp, "PCSCORES_K", Trim(ts%path)//"/k_bf/pcscores_k.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addradrec ) then
          Call PrintProfiles ( err, ts%data%data_k_bf%profiles_k_rec, "PROFILES_K_REC",  Trim(ts%path)//"/k_bf/profiles_k_rec.txt" )
          THROW(err.ne.0)
        else
          Call PrintProfiles ( err, ts%data%data_k_bf%profiles_k_pc, "PROFILES_K_PC",  Trim(ts%path)//"/k_bf/profiles_k_pc.txt" )
          THROW(err.ne.0)
        endif
      EndIf
    EndIf
  
    ts%defn%opts%verbose_checkinput_warnings = .false.
  
  EndIf
  
  
  If( ts%defn%do_k_tl ) Then
  
    Call rttov_k_tl(                                       &
      & ts%data%rttov_errorstatus,                         &
      & ts%data%chanprof,                                  &
      & ts%defn%opts,                                      &
      & ts%data%direct%profiles,                           &
      & ts%data%k_tl%profiles,                             &
      & ts%data%coefs,                                     &
      & ts%data%calcemis,                                  &
      & ts%data%direct%emissivity,                         &
      & ts%data%k_tl%emissivity,                           &
      & ts%data%direct%emissivity_out,                     &
      & ts%data%direct%transmission,                       &
      & ts%data%direct%radiance,                           &
      & pccomp         = ts%data%direct%pccomp,            &
      & profiles_k_rec = ts%data%data_k_tl%profiles_k_rec, &
      & profiles_k_pc  = ts%data%data_k_tl%profiles_k_pc,  &
      & channels_rec   = ts%data%channels_rec )
  
    If( ts%data%rttov_errorstatus /= 0 ) Then
      err = errorstatus_fatal
      THROW(err.ne.0)
    EndIf
      
    If( ts%defn%do_print ) Then
      Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE",  Trim(ts%path)//"/k_tl/radiance.txt", ts%defn%printallrad )
      THROW(err.ne.0)
      Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION",  Trim(ts%path)//"/k_tl/transmission.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/k_tl/emissivity_out.txt" )
      THROW(err.ne.0)
      Call PrintProfiles ( err, ts%data%k_tl%profiles, "PROFILES_K",  Trim(ts%path)//"/k_tl/profiles_k.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%k_tl%emissivity, "EMISSIVITY_K",  Trim(ts%path)//"/k_tl/emissivity_k.txt" )
      THROW(err.ne.0)
      If( ts%defn%opts%addpc ) Then
        Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/k_tl/pcscores.txt" )
        THROW(err.ne.0)
        Call PrintPCScores (err, ts%data%k_tl%pccomp, "PCSCORES_K", Trim(ts%path)//"/k_tl/pcscores_k.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addradrec ) then
          Call PrintProfiles ( err, ts%data%data_k_tl%profiles_k_rec, "PROFILES_K_REC",  Trim(ts%path)//"/k_tl/profiles_k_rec.txt" )
          THROW(err.ne.0)
        else
          Call PrintProfiles ( err, ts%data%data_k_tl%profiles_k_pc, "PROFILES_K_PC",  Trim(ts%path)//"/k_tl/profiles_k_pc.txt" )
          THROW(err.ne.0)
        endif
      EndIf
    EndIf
  
    ts%defn%opts%verbose_checkinput_warnings = .false.
  
  EndIf
  
  
  If( ts%defn%do_k_ad ) Then
  
    Call rttov_k_ad(                                       &
      & ts%data%rttov_errorstatus,                         &
      & ts%data%chanprof,                                  &
      & ts%defn%opts,                                      &
      & ts%data%direct%profiles,                           &
      & ts%data%k_ad%profiles,                             &
      & ts%data%coefs,                                     &
      & ts%data%calcemis,                                  &
      & ts%data%direct%emissivity,                         &
      & ts%data%k_ad%emissivity,                           &
      & ts%data%direct%emissivity_out,                     &
      & ts%data%direct%transmission,                       &
      & ts%data%direct%radiance,                           &
      & pccomp         = ts%data%direct%pccomp,            &
      & profiles_k_rec = ts%data%data_k_ad%profiles_k_rec, &
      & profiles_k_pc  = ts%data%data_k_ad%profiles_k_pc,  &
      & channels_rec   = ts%data%channels_rec )
  
    If( ts%data%rttov_errorstatus /= 0 ) Then
      err = errorstatus_fatal
      THROW(err.ne.0)
    EndIf
      
    If( ts%defn%do_print ) Then
      Call PrintRadiance (err, ts%data%direct%radiance, "RADIANCE",  Trim(ts%path)//"/k_ad/radiance.txt", ts%defn%printallrad )
      THROW(err.ne.0)
      Call PrintTransmission (err, ts%data%direct%transmission, "TRANSMISSION",  Trim(ts%path)//"/k_ad/transmission.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%direct%emissivity_out, "EMISSIVITY_OUT", Trim(ts%path)//"/k_ad/emissivity_out.txt" )
      THROW(err.ne.0)
      Call PrintProfiles ( err, ts%data%k_ad%profiles, "PROFILES_K",  Trim(ts%path)//"/k_ad/profiles_k.txt" )
      THROW(err.ne.0)
      Call PrintEmissivity (err, ts%data%k_ad%emissivity, "EMISSIVITY_K",  Trim(ts%path)//"/k_ad/emissivity_k.txt" )
      THROW(err.ne.0)
      If( ts%defn%opts%addpc ) Then
        Call PrintPCScores (err, ts%data%direct%pccomp, "PCSCORES", Trim(ts%path)//"/k_ad/pcscores.txt" )
        THROW(err.ne.0)
        Call PrintPCScores (err, ts%data%k_ad%pccomp, "PCSCORES_K", Trim(ts%path)//"/k_ad/pcscores_k.txt" )
        THROW(err.ne.0)
        If( ts%defn%opts%addradrec ) then
          Call PrintProfiles ( err, ts%data%data_k_ad%profiles_k_rec, "PROFILES_K_REC",  Trim(ts%path)//"/k_ad/profiles_k_rec.txt" )
          THROW(err.ne.0)
        else
          Call PrintProfiles ( err, ts%data%data_k_ad%profiles_k_pc, "PROFILES_K_PC",  Trim(ts%path)//"/k_ad/profiles_k_pc.txt" )
          THROW(err.ne.0)
        endif
      EndIf
    EndIf
  
    ts%defn%opts%verbose_checkinput_warnings = .false.
  
  EndIf
  
  If( ts%defn%do_taylor ) Then
    
    ! For IR sensors when addpc is false or the surface type is not sea,
    ! the calculated emissivity is independent of the profile and so
    ! will not reflect the perturbed profile. So we need to manually
    ! perturb the emissivity in this case.
    
    If (ts%data%coefs%coef%id_sensor == sensor_id_ir .or. &
        ts%data%coefs%coef%id_sensor == sensor_id_hi) Then
      Do i = 1, ts%defn%nchannels
        j = ts%data%chanprof(i)%prof
        If (.not. (ts%defn%opts%addpc .and. &
            ts%data%direct%profiles(j)%skin%surftype == surftype_sea)) Then
          
          ! Save the calculated emissivity for this channel and turn calcemis
          ! off so we can supply the perturbed emissivity to rttov_direct
          ts%data%direct%emissivity(i) = ts%data%direct%emissivity_out(i)
          ts%data%calcemis(i) = .false.
          
        EndIf
      EndDo
    EndIf
    
    Allocate(sumdir(niter,ts%defn%nprofiles), sumtl(niter,ts%defn%nprofiles))
    Do iter = 1, niter
      
      ! Scale the profile increments in place by a factor of 0.1 at each iteration
      If (iter > 1) Then 
        Call rttov_scale_profile_inc( ts%data%tl%profiles, 0.1_jprb )
        ts%data%tl%emissivity(:) = ts%data%tl%emissivity(:) * 0.1_jprb
      EndIf

      Call rttov_copy_prof(ts%data%direct%profiles_pert, ts%data%direct%profiles)
      Call rttov_add_prof(ts%data%direct%profiles_pert, ts%data%direct%profiles, ts%data%tl%profiles)
      
      ts%data%direct%emissivity_pert(:) = ts%data%direct%emissivity(:) + ts%data%tl%emissivity(:)

      Call rttov_direct(                       &
        & ts%data%rttov_errorstatus,           &
        & ts%data%chanprof,                    &
        & ts%defn%opts,                        &
        & ts%data%direct%profiles_pert,        &
        & ts%data%coefs,                       &
        & ts%data%calcemis,                    &
        & ts%data%direct%emissivity_pert,      &
        & ts%data%direct%emissivity_out,       &
        & ts%data%direct%transmission,         &
        & ts%data%direct%radiance,             &
        & pccomp       = ts%data%direct%pccomp,&
        & channels_rec = ts%data%channels_rec, &
        & traj         = traj )
    
      If( ts%data%rttov_errorstatus /= 0 ) Then
        err = errorstatus_fatal
        THROW(err.ne.0)
      EndIf
      
      j = 1
      Do i = 1, ts%defn%nprofiles
        sumdir(iter,i) = 0._jprb
        sumtl(iter,i) = 0._jprb
        If( ts%defn%opts%addpc ) Then
          If( ts%defn%opts%addradrec ) Then
            nchanrec = size(ts%data%channels_rec) / ts%defn%nprofiles
            j = (i-1)*nchanrec+1
            If( ts%defn%opts%switchrad ) Then
              sumdir(iter,i) = sumdir(iter,i) + sum(ts%data%direct%pccomp%bt_pccomp(j:j+nchanrec-1) - &
                                                    ts%data%direct%pccomp_saved%bt_pccomp(j:j+nchanrec-1))
              sumtl(iter,i)  = sumtl(iter,i)  + sum(ts%data%tl%pccomp%bt_pccomp(j:j+nchanrec-1))
            Else
              sumdir(iter,i) = sumdir(iter,i) + sum(ts%data%direct%pccomp%clear_pccomp(j:j+nchanrec-1) - &
                                                    ts%data%direct%pccomp_saved%clear_pccomp(j:j+nchanrec-1))
              sumtl(iter,i)  = sumtl(iter,i)  + sum(ts%data%tl%pccomp%clear_pccomp(j:j+nchanrec-1))
            EndIf
          Else
            npcscores = size(ts%data%direct%pccomp%pcscores) / ts%defn%nprofiles
            j = (i-1)*npcscores+1
            sumdir(iter,i) = sumdir(iter,i) + sum(ts%data%direct%pccomp%pcscores(j:j+npcscores-1) - &
                                                  ts%data%direct%pccomp_saved%pcscores(j:j+npcscores-1))
            sumtl(iter,i)  = sumtl(iter,i)  + sum(ts%data%tl%pccomp%pcscores(j:j+npcscores-1))
          EndIf
        Else
          Do While (ts%data%chanprof(j)%prof == i)
            sumdir(iter,i) = sumdir(iter,i) + ts%data%direct%radiance%bt(j) - ts%data%direct%radiance_saved%bt(j)
            sumtl(iter,i) = sumtl(iter,i) + ts%data%tl%radiance%bt(j)
            j = j + 1
            If( j > size(ts%data%chanprof) ) Exit
          EndDo
          If( j > size(ts%data%chanprof) ) Exit
        EndIf
      EndDo

    EndDo
    
    Do i = 1, ts%defn%nprofiles
      Print '(a)','Taylor test: profile, iteration, (H(x*)-H(x))/H(x*-x)'
      Do iter = 1, niter
        Print '(14x,i5,4x,i6,5x,7f20.16)',i,iter,sumdir(iter,i) * (10._jprb**(iter-1)) / sumtl(iter,i)
      EndDo
    EndDo
    
    Deallocate(sumdir, sumtl)
  EndIf
  
  ts%defn%do_print = .false.
  

CATCH
  End Subroutine

  
  Subroutine alloc_traj( err, ts, asw )
!
    Integer(Kind=jpim),      Intent(out)           :: err
    Type(rttov_test_struct), Intent(inout), Target :: ts
    Integer(Kind=jpim),      Intent(in)            :: asw
!
TRY

    If( ts%defn%do_direct ) Then                   
      call rttov_alloc_traj(                               &
          & err,                  ts%defn%nprofiles,       &
          & ts%defn%nchannels,    ts%defn%opts,            &
          & ts%defn%nlevels,      ts%data%coefs,           &
          & asw,                  traj = ts%data%traj )
      THROW(err.ne.0)
    EndIf
    
    If( ts%defn%do_tl ) Then                       
      call rttov_alloc_traj(                               &
          & err,                  ts%defn%nprofiles,       &
          & ts%defn%nchannels,    ts%defn%opts,            &
          & ts%defn%nlevels,      ts%data%coefs,           &
          & asw,                  traj_tl = ts%data%traj_tl )
      THROW(err.ne.0)
    EndIf
    
    If( ts%defn%do_ad ) Then                       
      call rttov_alloc_traj(                               &
          & err,                  ts%defn%nprofiles,       &
          & ts%defn%nchannels,    ts%defn%opts,            &
          & ts%defn%nlevels,      ts%data%coefs,           &
          & asw,                  traj_ad = ts%data%traj_ad )
      THROW(err.ne.0)
    EndIf
    
    If( ts%defn%do_k ) Then                        
      call rttov_alloc_traj(                               &
          & err,                  ts%defn%nprofiles,       &
          & ts%defn%nchannels,    ts%defn%opts,            &
          & ts%defn%nlevels,      ts%data%coefs,           &
          & asw,                  traj_k = ts%data%traj_k )
      THROW(err.ne.0)
    EndIf
    

CATCH

 
  End Subroutine

  Subroutine Setup( ts, path, err )
 !
    Type(rttov_test_struct), Intent(inout), Target :: ts
    Character*(*),           Intent(in)            :: path
    Integer(Kind=jpim),      Intent(out)           :: err
 !
    Integer(Kind=jpim), Pointer :: kchannels(:), & ! absolute channel number in coef file, full list of channels
                                   ichannels(:), & ! restricted list; each channel number appears once
                                   lchannels(:)    ! absolute channel number -> relative channel number
    Integer(Kind=jpim) :: i, j, lchannelmax, ichannelsmax 
    Integer(Kind=jpim) :: nprofiles1, nchannels1
    Integer(Kind=jpim) :: imult, k1, k2
    Integer(Kind=jpim) :: file_id_nml
    Type(rttov_test_defn) :: defn
    Namelist / coef_nml / defn
    Namelist / rttov_test_nml / defn
TRY

    call init_defn(defn)
    call rttov_get_lun(file_id_nml )

    Open( file_id_nml, file = Trim(path)//'/rttov_test.txt', iostat = err)
    THROWM(err.ne.0,"Cannot open "//Trim(path)//'/rttov_test.txt')
    Read( file_id_nml, nml = rttov_test_nml, iostat = err)
    THROWM(err.ne.0,"Cannot read "//Trim(path)//'/rttov_test.txt')
    Close( file_id_nml, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(path)//'/rttov_test.txt')


    Open( file_id_nml, file = Trim(path)//'/../in/coef.txt', iostat = err)
    THROWM(err.ne.0,"Cannot open "//Trim(path)//'/../in/coef.txt')
    Read( file_id_nml, nml = coef_nml, iostat = err)
    THROWM(err.ne.0,"Cannot read "//Trim(path)//'/../in/coef.txt')
    Close( file_id_nml, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(path)//'/../in/coef.txt')

    call rttov_put_lun (file_id_nml)

    defn%opts%ozone_data = defn%f_o3     .ne. ""
    defn%opts%co2_data   = defn%f_co2    .ne. ""
    defn%opts%n2o_data   = defn%f_n2o    .ne. ""
    defn%opts%co_data    = defn%f_co     .ne. ""
    defn%opts%ch4_data   = defn%f_ch4    .ne. ""
    defn%opts%clw_data   = defn%f_clw    .ne. ""
    defn%opts%addaerosl  = defn%f_aerosl .ne. ""
    defn%opts%addclouds  = defn%f_cloud  .ne. "" &
                     .and. defn%f_cfrac  .ne. ""
    defn%opts%addpc      = defn%f_pcscores     .ne. ""
    defn%opts%addradrec  = defn%f_channels_rec .ne. "" .and. defn%opts%addpc

    nprofiles1 = defn%nprofiles
    nchannels1 = defn%nchannels

    defn%nprofiles = defn%mult * defn%nprofiles
    defn%nchannels = defn%mult * defn%nchannels

    ts%defn = defn
    ts%path = path
    
    if(ts%defn%do_taylor) then
      if( ts%defn%do_ad   .or. &
          ts%defn%do_k    .or. &
          ts%defn%do_k_tl .or. &
          ts%defn%do_k_ad .or. &
          ts%defn%do_k_bf ) then
        err = errorstatus_fatal
        THROWM(err.ne.0,"Taylor test must be run alone")
      endif
      ts%defn%do_direct = .true. 
      ts%defn%do_tl     = .true.
    endif
    
! read input data

    if (ts%defn%opts%addpc) then
      if (ts%defn%opts%addradrec) then
        allocate(ts%data%channels_rec(ts%defn%nchannels_rec), stat = err)
        THROW(err.ne.0)
        call reada( err, Trim(ts%path)//'/../in/channels_rec.txt',  n1 = ts%data%channels_rec, foe = .true._jplm )
        THROW(err.ne.0)
      endif
    endif

    allocate(                                       &
      ts%data%chanprof( defn%nchannels ),           &
      ts%data%calcemis( defn%nchannels ),           &
      kchannels( nchannels1 ),                      &
      stat = err )
    THROWM(err.ne.0,"Cannot allocate ts%data")

    call reada( err, Trim(ts%path)//'/../in/channels.txt',  n1 = kchannels,                       foe = .true._jplm )
    THROW(err.ne.0)
    call reada( err, Trim(ts%path)//'/../in/lprofiles.txt', n1 = ts%data%chanprof(1:nchannels1)%prof, foe = .true._jplm )
    THROW(err.ne.0)

!

    call reada( err, Trim(ts%path)//'/../in/flags.txt', addinterp = ts%defn%opts%addinterp )
    THROW(err.ne.0)
    If(.Not. ts%defn%opts%addinterp .and. ts%defn%opts%lgradp)Then
      ts%defn%opts%lgradp = .False.
    End If

    if (ts%defn%opts%addpc) then
      call reada( err, Trim(ts%path)//'/../in/pcscores.txt', npcscores = ts%defn%npcscores, &
                  ipcreg = ts%defn%opts%ipcreg, foe = .true._jplm )
      THROW(err.ne.0)
    endif

! channels indices calculations

    lchannelmax = 0_jpim
    Do i = 1, nchannels1
      If( kchannels( i ) .gt. lchannelmax ) lchannelmax = kchannels( i )
    EndDo
    
    Allocate( lchannels( lchannelmax ), stat = err )
    THROWM(err.ne.0,"Cannot allocate lchannels")

    lchannels = 0_jpim
    Do i = 1, nchannels1
      If( lchannels( kchannels( i ) ) .eq. 0_jpim ) &
        lchannels( kchannels( i ) ) = i
    EndDo
    ichannelsmax = 0_jpim
    Do i = 1, lchannelmax
      If( lchannels( i ) .gt. 0_jpim ) ichannelsmax = ichannelsmax + 1
    EndDo
    
    Allocate( ichannels( ichannelsmax ), stat = err )
    THROWM(err.ne.0,"Cannot allocate ichannels")

    j = 0_jpim
    Do i = 1, lchannelmax
      If( lchannels( i ) .gt. 0_jpim ) Then
        j = j + 1_jpim
        ichannels( j ) = i
      EndIf
    EndDo
   
!
!
! read coefficients

    
    If( ts%defn%lallchans ) Then
      Call rttov_read_coefs( &
        & err, ts%data%coefs, ts%defn%opts, &
        & file_coef   = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_coef),   &
        & form_coef   = Trim(ts%defn%coef_format),                              &
        & file_scaer  = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_scat),   &
        & form_scaer  = Trim(ts%defn%coef_format),                              &
        & file_sccld  = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_optp),   &
        & form_sccld  = Trim(ts%defn%coef_format),                              &
        & file_pccoef = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_pccomp), &
        & form_pccoef = Trim(ts%defn%coef_format))

      THROW(err.ne.0)
    Else
      Call rttov_read_coefs( &
        & err, ts%data%coefs, ts%defn%opts,        &
        & channels     = ichannels(1:ichannelsmax), &
        & channels_rec = ts%data%channels_rec,      &
        & file_coef    = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_coef),   &
        & form_coef    = Trim(ts%defn%coef_format),                              &
        & file_scaer   = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_scat),   &
        & form_scaer   = Trim(ts%defn%coef_format),                              &
        & file_sccld   = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_optp),   &
        & form_sccld   = Trim(ts%defn%coef_format),                              &
        & file_pccoef  = Trim(ts%defn%coef_prefix)//'/'//Trim(ts%defn%f_pccomp), &
        & form_pccoef  = Trim(ts%defn%coef_format))

      THROW(err.ne.0)
    EndIf
  
    If( ts%defn%lallchans ) Then
      ts%data%chanprof(1:nchannels1)%chan = kchannels
    Else
!
! Renumber all channels
!
      ts%data%chanprof(1:nchannels1)%chan = lchannels(kchannels)
      If (ts%defn%opts%addpc) Then
        If (ts%defn%opts%addradrec) Then
          Do i = 1, ts%defn%nchannels_rec
            ts%data%channels_rec( i ) = i
          EndDo
        EndIf
      EndIf
    EndIf

!
!

    Call rttov_init_coefs ( err, ts%defn%opts, ts%data%coefs )
    THROW(err.ne.0)



  
! alloc data
  
    call alloc_data( err, ts, ts%defn%nprofiles, ts%data%direct, do_taylor=ts%defn%do_taylor )
      
    If( ts%defn%do_tl    ) Then
      call alloc_data( err, ts, ts%defn%nprofiles, ts%data%tl )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_ad    ) Then
      call alloc_data( err, ts, ts%defn%nprofiles, ts%data%ad,   do_ad=.true._jplm )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k     ) Then
      call alloc_data( err, ts, ts%defn%nchannels, ts%data%k,    ts%data%data_k )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_bf  ) Then
      call alloc_data( err, ts, ts%defn%nchannels, ts%data%k_bf, ts%data%data_k_bf )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_tl  ) Then
      call alloc_data( err, ts, ts%defn%nchannels, ts%data%k_tl, ts%data%data_k_tl )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_ad  ) Then
      call alloc_data( err, ts, ts%defn%nchannels, ts%data%k_ad, ts%data%data_k_ad )
      THROW(err.ne.0)
    endif
   

!
! emissivity
!
    ts%data%calcemis          = .FALSE.
    ts%data%direct%emissivity = 1._jprb
    call reada( err, Trim(ts%path)//'/../in/calcemis.txt',    l1 = ts%data%calcemis(1:nchannels1) )
    THROW(err.ne.0)
    Call reada( err, Trim(ts%path)//'/../in/emissivity.txt',  r1 = ts%data%direct%emissivity(1:nchannels1) )
    THROW(err.ne.0)


!
! replicate emissivity data
!
    Do imult = 2, ts%defn%mult
      k1 = 1 + ( imult - 1 ) * nchannels1
      k2 = k1 + nchannels1 - 1
      ts%data%direct%emissivity(k1:k2) = ts%data%direct%emissivity(1:nchannels1)
      ts%data%chanprof(k1:k2)%chan     = ts%data%chanprof(1:nchannels1)%chan
      ts%data%chanprof(k1:k2)%prof     = ts%data%chanprof(1:nchannels1)%prof + ( imult - 1 ) * nprofiles1
      ts%data%calcemis(k1:k2)          = ts%data%calcemis(1:nchannels1)
    EndDo


    Call read_profile_data( ts, err )
    THROW(err.ne.0)
  
    Deallocate( kchannels, lchannels, ichannels, stat = err )
    THROWM(err.ne.0,"Deallocation of kchannels, lchannels, ... failed")
  
  
    If( ts%defn%ltemp ) Then
      Call alloc_traj( err, ts, 1_jpim )
      THROW(err.ne.0)
    endif

    
    If( ts%defn%do_tl ) Then
      call rttov_make_profile_inc( ts%data%tl%profiles, ts%data%direct%profiles,&
       &     ts%defn%opts )

      ts%data%tl%emissivity(:) = 0._jprb
      Do i = 1, ts%defn%nchannels
        j = ts%data%chanprof(i)%prof
        
        If (ts%data%coefs%coef%id_sensor == sensor_id_mw .or. &
            ts%data%coefs%coef%id_sensor == sensor_id_po) Then
          
          ! For MW sensors rttov_calcemis_mw_tl calculates the TL if calcemis(:) is true
          If (.not. ts%data%calcemis(i)) Then
            ts%data%tl%emissivity(i) = -0.01_jprb * (Modulo(j,3_jpim)+1)
          End If
          
        Else  
          
          ! For IR sensors rttov_calcemis_ir_tl only calculates the TL if addpc is true 
          ! and the surface type is sea.
          If (.not. (ts%defn%opts%addpc .and. &
              ts%data%direct%profiles(j)%skin%surftype == surftype_sea)) Then
            ts%data%tl%emissivity(i) = -0.01_jprb * (Modulo(j,3_jpim)+1)
          End If
        
        End If
        
      End Do
      
      If( ts%defn%scale_inc /= 1.0_jprb ) Then
        Call rttov_scale_profile_inc( ts%data%tl%profiles, ts%defn%scale_inc )
        ts%data%tl%emissivity(:) = ts%data%tl%emissivity(:) * ts%defn%scale_inc
      End If
    End If

    If( ts%defn%do_ad ) Then
      If( ts%defn%opts%addpc )Then
        Call rttov_make_pccomp_inc( ts%data%ad%pccomp_saved, ts%defn%opts ) 
        If( ts%defn%scale_inc /= 1.0_jprb ) &
          Call rttov_scale_pccomp_inc( ts%data%ad%pccomp_saved, ts%defn%scale_inc, & 
         &                   ts%defn%opts ) 
      Else
        call rttov_make_radiance_inc( ts%data%coefs%coef, ts%data%ad%radiance_saved, &
           ts%data%chanprof(:)%chan, ts%defn%nchannels, ts%defn%opts )
    
        If( ts%defn%scale_inc /= 1.0_jprb ) &
          Call rttov_scale_radiance_inc( ts%data%ad%radiance_saved, &
            ts%defn%scale_inc, ts%defn%opts )
      End If
    End If
    
    If (ts%data%coefs%coef%inczeeman) Then
      print *,"Zeeman coefficient file with    Be   cosbk"
      Do i = 1, nprofiles1
        print "(24x,i4,2f8.4)",i,ts%data%direct%profiles(i)%be,ts%data%direct%profiles(i)%cosbk
      EndDo
    EndIf
CATCH

  End Subroutine  

  Subroutine read_profile_data( ts, err )
    Type(rttov_test_struct), Intent(inout), Target :: ts
    Integer(Kind=jpim),      Intent(out)           :: err
!
    Real(Kind=jprb)  :: becosbk(2)
    Type(rttov_test_defn), Pointer :: defn
    Type(rttov_input_data), Pointer :: direct
    Integer(Kind=jpim) :: iprof
    Integer(Kind=jpim) :: kprof
    Integer(Kind=jpim) :: imult
    Character*3 :: sprof
    Character*256 :: in_dir
    Integer(Kind=jpim) :: nprofiles1
    Logical(Kind=jplm) :: foe
    Logical(Kind=jplm) :: file_exists
!
TRY


    defn => ts%defn
    direct => ts%data%direct

    nprofiles1 = ts%defn%nprofiles / ts%defn%mult



    Do iprof = 1, ts%defn%nprofiles / ts%defn%mult


      foe = iprof .eq. 1

      write( sprof, '(i3.3)' ) iprof
      in_dir =  Trim(ts%path)//'/../in/profiles/'//sprof


      call reada( err, Trim(in_dir)//'/ground/skin.txt', k0 = direct%profiles(iprof)%skin, &
                  foe = foe, k0_a = direct%profiles(1)%skin )
      THROW(err.ne.0)
      call reada( err, Trim(in_dir)//'/ground/s2m.txt',  s0 = direct%profiles(iprof)%s2m,  &
                  foe = foe, s0_a = direct%profiles(1)%s2m )
      THROW(err.ne.0)
      call reada( err, Trim(in_dir)//'/ground/elevation.txt', &
                  elevation = direct%profiles(iprof)%elevation, &
                  foe = foe, elevation_a = direct%profiles(1)%elevation )
      THROW(err.ne.0)
      call reada( err, Trim(in_dir)//'/angles.txt', &
        zenangle    = direct%profiles(iprof)%zenangle,    azangle    = direct%profiles(iprof)%azangle,    &
        sunzenangle = direct%profiles(iprof)%sunzenangle, sunazangle = direct%profiles(iprof)%sunazangle, &
        latitude    = direct%profiles(iprof)%latitude,                                                    &
        foe = foe,                          &
        zenangle_a    = direct%profiles(1)%zenangle,    azangle_a    = direct%profiles(1)%azangle,    &
        sunzenangle_a = direct%profiles(1)%sunzenangle, sunazangle_a = direct%profiles(1)%sunazangle, &
        latitude_a    = direct%profiles(1)%latitude                                                   &
         )
      THROW(err.ne.0)
      call reada( err, Trim(in_dir)//'/atm/cloud0.txt', &
        ctp = direct%profiles(iprof)%ctp, cfraction = direct%profiles(iprof)%cfraction, &
        ctp_a = direct%profiles(1)%ctp, cfraction_a = direct%profiles(1)%cfraction, &
        foe = foe )
      THROW(err.ne.0)
      call reada( err, Trim(in_dir)//'/atm/aerosli.txt', &
                  idg = direct%profiles(iprof)%idg, ish = direct%profiles(iprof)%ish, &
                  idg_a = direct%profiles(1)%idg, ish_a = direct%profiles(1)%ish, &
                  foe = foe )
      THROW(err.ne.0)
      Call reada( err, Trim(in_dir)//'/atm/p.txt',  r1 = direct%profiles(iprof)%p, foe = foe )
      THROW(err.ne.0)
      Call reada( err, Trim(in_dir)//'/atm/t.txt',  r1 = direct%profiles(iprof)%t, foe = foe )
      THROW(err.ne.0)
      Call reada( err, Trim(in_dir)//'/atm/q.txt',  r1 = direct%profiles(iprof)%q, foe = foe )
      THROW(err.ne.0)

      becosbk(1) = direct%profiles(1)%be  
      becosbk(2) = direct%profiles(1)%cosbk

      Call reada( err, Trim(in_dir)//'/be.txt', r1 = becosbk, foe = foe )
      THROW(err.ne.0)

      direct%profiles(iprof)%nlevels     = ts%defn%nlevels

      direct%profiles(iprof)%be    = becosbk(1)
      direct%profiles(iprof)%cosbk = becosbk(2)


      If( defn%opts%ozone_data ) then
        Call reada( err, Trim(in_dir)//'/atm/o3.txt',     &
          r1 = direct%profiles(iprof)%o3,  foe = foe )
        THROW(err.ne.0)
      endif
      If( defn%opts%co2_data   ) then
        Call reada( err, Trim(in_dir)//'/atm/co2.txt',    &
          r1 = direct%profiles(iprof)%co2, foe = foe ) 
        THROW(err.ne.0)
      endif
      If( defn%opts%n2o_data   ) then
        Call reada( err, Trim(in_dir)//'/atm/n2o.txt',    &
          r1 = direct%profiles(iprof)%n2o, foe = foe ) 
        THROW(err.ne.0)
      endif
      If( defn%opts%co_data    ) then
        Call reada( err, Trim(in_dir)//'/atm/co.txt',     &
          r1 = direct%profiles(iprof)%co,  foe = foe ) 
        THROW(err.ne.0)
      endif
      If( defn%opts%ch4_data   ) then
        Call reada( err, Trim(in_dir)//'/atm/ch4.txt',    &
          r1 = direct%profiles(iprof)%ch4, foe = foe ) 
        THROW(err.ne.0)
      endif
      If( defn%opts%clw_data   ) then
        Call reada( err, Trim(in_dir)//'/atm/clw.txt',    &
          r1 = direct%profiles(iprof)%clw, foe = foe ) 
        THROW(err.ne.0)
      endif
      If( defn%opts%addaerosl  ) then
        Call reada( err, Trim(in_dir)//'/atm/aerosl.txt', &
          r2 = direct%profiles(iprof)%aerosols, foe = foe )  
        THROW(err.ne.0)
      endif
      If( defn%opts%addclouds ) Then
        Call reada( err, Trim(in_dir)//'/atm/cfrac.txt', &
          r2 = direct%profiles(iprof)%cfrac, foe = foe )  
        THROW(err.ne.0)
        Call reada( err, Trim(in_dir)//'/atm/cloud.txt', &
          r2 = direct%profiles(iprof)%cloud, foe = foe )  
        THROW(err.ne.0)
        Inquire(file=Trim(in_dir)//'/atm/icede.txt', exist=file_exists)
        If (file_exists) Then
          Call reada( err, Trim(in_dir)//'/atm/icede.txt', &
            r1 = direct%profiles(iprof)%icede, foe = foe )  
          THROW(err.ne.0)
        End If
      EndIf

!
! if multiplicity > 1, then replicate profile data
!
      Do imult = 2, ts%defn%mult
        kprof = iprof + (imult-1) * nprofiles1
        Call rttov_copy_prof( direct%profiles(kprof:kprof), direct%profiles(iprof:iprof) )
      EndDo

    EndDo

CATCH
  End Subroutine

  Subroutine Cleanup( ts, err )
!
    Type(rttov_test_struct), Intent(inout), Target :: ts
    Integer(Kind=jpim),      Intent(out)           :: err
!
TRY

    deallocate( ts%data%chanprof, ts%data%calcemis, stat = err )  
    THROWM(err.ne.0,"Deallocation of ts%data failed")

    if(ts%defn%opts%addpc) then
     if(ts%defn%opts%addradrec) then
       deallocate( ts%data%channels_rec, stat = err)
       THROW(err.ne.0)
     endif
    endif

    If( ts%defn%ltemp ) Then
      Call alloc_traj( err, ts, 0_jpim )
      THROW(err.ne.0)
    endif

    If( ts%defn%do_tl    ) Then
      call dealloc_data( err, ts, ts%defn%nprofiles, ts%data%tl )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_ad    ) Then
      call dealloc_data( err, ts, ts%defn%nprofiles, ts%data%ad,   do_ad=.true._jplm )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k     ) Then
      call dealloc_data( err, ts, ts%defn%nchannels, ts%data%k,    ts%data%data_k )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_bf  ) Then
      call dealloc_data( err, ts, ts%defn%nchannels, ts%data%k_bf, ts%data%data_k_bf )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_tl  ) Then
      call dealloc_data( err, ts, ts%defn%nchannels, ts%data%k_tl, ts%data%data_k_tl )
      THROW(err.ne.0)
    endif
    If( ts%defn%do_k_ad  ) Then
      call dealloc_data( err, ts, ts%defn%nchannels, ts%data%k_ad, ts%data%data_k_ad )
      THROW(err.ne.0)
    endif

    call dealloc_data( err, ts, ts%defn%nprofiles, ts%data%direct, do_taylor=ts%defn%do_taylor )
    THROW(err.ne.0)
    
    call rttov_dealloc_coefs( err, ts%data%coefs )
    THROW(err.ne.0)

CATCH
    
  End Subroutine
  

  Subroutine reada( err, f,                                          &
    r1, r1_a, r2, r2_a, n1, n1_a, l1, l1_a, k0, k0_a, s0, s0_a,      &
    zenangle,   azangle,   sunzenangle,   sunazangle,   elevation,   latitude,   &
    zenangle_a, azangle_a, sunzenangle_a, sunazangle_a, elevation_a, latitude_a, &
    idg,   ish,                                                          &
    idg_a, ish_a,                                                        &
    ctp,   cfraction,                                                    &
    ctp_a, cfraction_a,                                                  &
    addinterp,                                                           &
    addinterp_a,                                                         &
    ipcreg,     npcscores,                                               &
    foe                                                                  &
  )
    Integer(Kind=jpim), Intent(out)   :: err
    Character*(*),      Intent(in)    :: f
    Real(Kind=jprb),    Intent(inout), Optional :: r1(:),   r2(:,:)
    Real(Kind=jprb),    Intent(in),    Optional :: r1_a(:), r2_a(:,:)
    Integer(kind=jpim), Intent(inout), Optional :: n1(:)
    Integer(kind=jpim), Intent(in),    Optional :: n1_a(:)
    Logical(Kind=jplm), Intent(inout), Optional :: l1(:)
    Logical(Kind=jplm), Intent(in),    Optional :: l1_a(:)
    Type(sskin_type),   Intent(inout), Optional :: k0
    Type(sskin_type),   Intent(in),    Optional :: k0_a
    Type(s2m_type),     Intent(inout), Optional :: s0
    Type(s2m_type),     Intent(in),    Optional :: s0_a
    Real(Kind=jprb),    Intent(inout), Optional :: zenangle,   azangle,   sunzenangle,   sunazangle,   elevation,   latitude
    Real(Kind=jprb),    Intent(in),    Optional :: zenangle_a, azangle_a, sunzenangle_a, sunazangle_a, elevation_a, latitude_a
    Real(Kind=jprb),    Intent(inout), Optional :: ctp,   cfraction
    Real(Kind=jprb),    Intent(in),    Optional :: ctp_a, cfraction_a
    Integer(Kind=jpim), Intent(inout), Optional :: idg,   ish
    Integer(Kind=jpim), Intent(in),    Optional :: idg_a, ish_a
    Logical(Kind=jplm), Intent(inout), Optional :: addinterp
    Logical(Kind=jplm), Intent(in),    Optional :: addinterp_a
    Integer(Kind=jpim), Intent(inout), Optional :: ipcreg
    Integer(Kind=jpim), Intent(inout), Optional :: npcscores
!
    Logical(Kind=jplm), Intent(in),    Optional :: foe ! fail on data not present
!
    Integer(Kind=jpim) :: ipcreg1, npcscores1
    Namelist / pcscores / ipcreg, npcscores
    Namelist / flags / addinterp
    Namelist / angles / zenangle, azangle, sunzenangle, sunazangle, latitude
    Namelist / s2m / s0
    Namelist / aerosli / idg, ish
    Namelist / cloud / ctp, cfraction
    Namelist / elev / elevation
    Namelist / skin / k0
!
    Logical(Kind=jplm) :: foe1
    Integer(Kind=jpim) :: file_id
!
TRY

    foe1 = .false.
    If( Present(foe) ) foe1 = foe
    

    call rttov_get_lun (file_id )
    If( foe1 ) Then
      Open( file_id, file = Trim( f ), form = 'formatted', status = 'old', iostat = err )
      THROWM(err.ne.0,"Cannot open "//Trim( f ))
    Else
      Open( file_id, file = Trim( f ), form = 'formatted', status = 'old', iostat = err )
      If( err .ne. 0 ) Then
        If( Present( r1 ) .and. Present( r1_a ) ) Then
          r1 = r1_a
        Else If( Present( r2 ) .and. Present( r2_a ) ) Then
          r2 = r2_a
        Else If( Present( l1 ) .and. Present( l1_a ) ) Then
          l1 = l1_a
        Else If( Present( n1 ) .and. Present( n1_a ) ) Then
          n1 = n1_a
        Else If( Present( k0 ) .and. Present( k0_a ) ) Then
          k0 = k0_a
        Else If( Present( elevation ) .and. Present( elevation_a ) ) Then
          elevation = elevation_a
        Else If( Present( s0 ) .and. Present( s0_a ) ) Then
          s0 = s0_a
        Else If( Present( zenangle ) .and. Present( zenangle_a ) ) Then
          zenangle    = zenangle_a
          azangle     = azangle_a
          sunzenangle = sunzenangle_a
          sunazangle  = sunazangle_a
          latitude    = latitude_a
        Else If( Present( ish ) .and. Present( ish_a ) ) Then
          ish = ish_a
          idg = idg_a
        Else If( Present( ctp ) .and. Present( ctp_a ) ) Then
          ctp       = ctp_a
          cfraction = cfraction_a
        Else If( Present( addinterp ) .and. Present( addinterp_a ) ) Then
          addinterp = addinterp_a
        Else
          THROWM(err.ne.0,"Cannot open "//Trim( f ))
        EndIf
        err = 0
      endif
    Endif
    
    
    If( Present( r1 ) ) Then
      Read( file_id, *, iostat = err ) r1
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( r2 ) ) Then
      Read( file_id, *, iostat = err ) r2
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( l1 ) ) Then
      Read( file_id, *, iostat = err ) l1
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( n1 ) ) Then
      Read( file_id, *, iostat = err ) n1
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( k0 ) ) Then
      Read( file_id, nml = skin, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( elevation ) ) Then
      Read( file_id, nml = elev, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( s0 ) ) Then
      Read( file_id, nml = s2m, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( zenangle ) ) Then
      Read( file_id, nml = angles, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( ish ) ) Then
      Read( file_id, nml = aerosli, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( ctp ) ) Then
      Read( file_id, nml = cloud, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( addinterp ) ) Then
      Read( file_id, nml = flags, iostat = err ) 
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
    Else If( Present( ipcreg ) .and. Present( npcscores ) ) Then
      ipcreg1 = ipcreg
      npcscores1 = npcscores
      Read( file_id, nml = pcscores, iostat = err )
      THROWM(err.ne.0,"Cannot read from "//Trim( f ))
      If( ipcreg1 .gt. 0 ) ipcreg = ipcreg1
      If( npcscores1 .gt. 0 ) npcscores = npcscores1
    EndIf
    
    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim( f ))
    call rttov_put_lun (file_id)
    
CATCH
  End Subroutine
  
  
  
  Subroutine alloc_data( err, ts, nprofiles, dat, dat_k, do_ad, do_taylor )
!
    Integer(Kind=jpim),      Intent(out)   :: err
    Type(rttov_test_struct), Intent(in)    :: ts
    Integer(Kind=jpim),      Intent(in)    :: nprofiles
    Type(rttov_input_data),  Intent(inout) :: dat
    Type(rttov_k_input_data),Intent(inout), Optional :: dat_k
    Logical(Kind=jplm),      Intent(in),    Optional :: do_ad
    Logical(Kind=jplm),      Intent(in),    Optional :: do_taylor
!
    Logical(Kind=jplm) :: ldo_ad, ldo_taylor
!
TRY
    ldo_ad = .false._jplm
    if (present(do_ad)) ldo_ad = do_ad
    ldo_taylor = .false._jplm
    if (present(do_taylor)) ldo_taylor = do_taylor
    
    allocate(                                                               &
      dat%emissivity( ts%defn%nchannels ),                                  &
      dat%emissivity_out( ts%defn%nchannels ),                              &
      dat%profiles( nprofiles ),                                            &
      dat%transmission%tau_levels( ts%defn%nlevels, ts%defn%nchannels ),    &
      dat%transmission%tau_total( ts%defn%nchannels ), stat = err )
    THROWM(err.ne.0,"Cannot allocate dat")
    
    if( ldo_taylor ) then
      allocate( dat%profiles_pert( nprofiles ),                      &
                dat%emissivity_pert( ts%defn%nchannels ), stat = err )
      THROWM(err.ne.0,"Cannot allocate dat")
    end if    
   
    dat%emissivity              = 0._jprb
    dat%emissivity_out          = 0._jprb
    dat%transmission%tau_levels = 0._jprb
    dat%transmission%tau_total  = 0._jprb
   
    call rttov_alloc_prof( err, nprofiles, dat%profiles, ts%defn%nlevels, ts%defn%opts, 1_jpim, & 
                           & init = .true._jplm, coefs = ts%data%coefs )
    THROW(err.ne.0)

    call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance, ts%defn%nlevels-1_jpim, 1_jpim, init=.true._jplm )
    THROW(err.ne.0)

    if(ldo_taylor) then
      call rttov_alloc_prof( err, ts%defn%nprofiles, dat%profiles_pert, ts%defn%nlevels, ts%defn%opts, 1_jpim, & 
                           & init = .true._jplm, coefs = ts%data%coefs )
      THROW(err.ne.0)
      dat%emissivity_pert(:) = 0._jprb
      
      if(ts%defn%opts%addpc)then
        if(ts%defn%opts%addradrec)then
          call rttov_alloc_pccomp( err, dat%pccomp_saved, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, &
                                 & init = .true._jplm, nchannels_rec = ts%defn%nchannels_rec * ts%defn%nprofiles )
        else
          call rttov_alloc_pccomp( err, dat%pccomp_saved, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, &
                                 & init = .true._jplm )
        endif
      else
        call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance_saved, ts%defn%nlevels-1_jpim, 1_jpim )
      endif
      THROW(err.ne.0)
    end if
    
    if(ts%defn%opts%addpc)then
      if(ts%defn%opts%addradrec)then
        call rttov_alloc_pccomp( err, dat%pccomp, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, init = .true._jplm, &
                               & nchannels_rec = ts%defn%nchannels_rec * ts%defn%nprofiles )
      else
        call rttov_alloc_pccomp( err, dat%pccomp, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, init = .true._jplm )
      endif
      THROW(err.ne.0)
    endif

    if( ldo_ad ) then
      if( ts%defn%opts%addpc ) then
        if(ts%defn%opts%addradrec)then
          call rttov_alloc_pccomp( err, dat%pccomp_saved, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, &
                                 & init = .true._jplm, nchannels_rec = ts%defn%nchannels_rec * ts%defn%nprofiles )
        else
          call rttov_alloc_pccomp( err, dat%pccomp_saved, ts%defn%npcscores * ts%defn%nprofiles, 1_jpim, &
                                 & init = .true._jplm )
        endif
      else
        call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance_saved, ts%defn%nlevels-1_jpim, 1_jpim, &
                            & init=.true._jplm )
      end if
      THROW(err.ne.0)
    end if
    
    if(present(dat_k)) then
      if(ts%defn%opts%addpc) then
        if(ts%defn%opts%addradrec) then

          allocate(dat_k%profiles_k_rec( ts%defn%nchannels_rec * ts%defn%nprofiles ), stat = err)
          THROW(err.ne.0)
          call rttov_alloc_prof( err, ts%defn%nchannels_rec * ts%defn%nprofiles, dat_k%profiles_k_rec, ts%defn%nlevels, &
           & ts%defn%opts, 1_jpim, init = .true._jplm, coefs = ts%data%coefs )
          THROW(err.ne.0)

        else

          allocate(dat_k%profiles_k_pc( ts%defn%npcscores * ts%defn%nprofiles ), stat = err)
          THROW(err.ne.0)
          call rttov_alloc_prof( err, ts%defn%npcscores * ts%defn%nprofiles, &
           & dat_k%profiles_k_pc, &
           & ts%defn%nlevels, ts%defn%opts, 1_jpim, init = .true._jplm, coefs = ts%data%coefs )
          THROW(err.ne.0)

        endif
      endif

    endif

CATCH
     
  End Subroutine
  
  Subroutine dealloc_data( err, ts, nprofiles, dat, dat_k, do_ad, do_taylor )
!
    Integer(Kind=jpim),      Intent(out)   :: err
    Type(rttov_test_struct), Intent(in)    :: ts
    Integer(Kind=jpim),      Intent(in)    :: nprofiles
    Type(rttov_input_data),  Intent(inout) :: dat
    Type(rttov_k_input_data),Intent(inout), Optional :: dat_k
    Logical(Kind=jplm),      Intent(in),    Optional :: do_ad
    Logical(Kind=jplm),      Intent(in),    Optional :: do_taylor
!
    Logical(Kind=jplm) :: ldo_ad, ldo_taylor
!
TRY
    ldo_ad = .false._jplm
    if (present(do_ad)) ldo_ad = do_ad
    ldo_taylor = .false._jplm
    if (present(do_taylor)) ldo_taylor = do_taylor

    call rttov_alloc_prof( err, nprofiles, dat%profiles, ts%defn%nlevels, ts%defn%opts, 0_jpim )
    THROW(err.ne.0)
    call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance, ts%defn%nlevels-1_jpim, 0_jpim )
    THROW(err.ne.0)

    if (ldo_taylor) then
      call rttov_alloc_prof( err, nprofiles, dat%profiles_pert, ts%defn%nlevels, ts%defn%opts, 0_jpim )
      THROW(err.ne.0)
      if( ts%defn%opts%addpc ) then
        call rttov_alloc_pccomp (err, dat%pccomp_saved, ts%defn%npcscores, 0_jpim)
      else
        call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance_saved, ts%defn%nlevels-1_jpim, 0_jpim )
      end if
      THROW(err.ne.0)
    end if
    
    deallocate(                                      &
      dat%emissivity,                                &
      dat%emissivity_out,                            &
      dat%profiles,                                  &
      dat%transmission % tau_levels,                 &
      dat%transmission % tau_total, stat = err )
    THROWM(err.ne.0,"Deallocation of dat failed")

    if( ldo_taylor ) then
      deallocate( dat%profiles_pert,              &
                  dat%emissivity_pert, stat = err )
      THROWM(err.ne.0,"Cannot deallocate dat")
    end if  
    
    if(ts%defn%opts%addpc)then
      call rttov_alloc_pccomp (err, dat%pccomp, ts%defn%npcscores, 0_jpim)
      THROW(err.ne.0)
    endif

    if( ldo_ad ) then
      if( ts%defn%opts%addpc ) then
        call rttov_alloc_pccomp (err, dat%pccomp_saved, ts%defn%npcscores, 0_jpim)
      else
        call rttov_alloc_rad( err, ts%defn%nchannels, dat%radiance_saved, ts%defn%nlevels-1_jpim, 0_jpim )
      end if
      THROW(err.ne.0)
    end if
    
    if(present(dat_k)) then
      if(ts%defn%opts%addpc) then
        if(ts%defn%opts%addradrec) then

          call rttov_alloc_prof( err, 1_jpim, dat_k%profiles_k_rec, ts%defn%nlevels, ts%defn%opts, 0_jpim)
          THROW(err.ne.0)

          deallocate(dat_k%profiles_k_rec, stat = err)
          THROW(err.ne.0)

        else

          call rttov_alloc_prof( err, ts%defn%npcscores, dat_k%profiles_k_pc, ts%defn%nlevels, ts%defn%opts, 0_jpim )
          THROW(err.ne.0)

          deallocate(dat_k%profiles_k_pc, stat = err)
          THROW(err.ne.0)

        endif
      endif

    endif

CATCH
   
  End Subroutine

! Print routines  
  
  Subroutine PrintProfiles( err, profiles, name, f )
    Use rttov_chain, only : chain, chain_profile_type, print_chain, free_chain

    Integer(Kind=jpim), Intent(out) :: err
    Type(profile_type), Intent(in), Target :: profiles(:) ! target because of chain_profile_type
    Character*(*),      Intent(in) :: name, f
    
    Type(chain) :: chain_prof
    Integer(Kind=jpim) :: file_id

TRY
    
    call rttov_get_lun (file_id )
    Open( file_id, file = f, form = 'formatted', iostat = err )
    THROWM(err.ne.0,"Cannot open "//Trim(f))
    
    Call chain_profile_type( err, chain_prof, name, a1 = profiles )
    THROWM(err.ne.0,"Cannot chain profiles")
    
    Call print_chain( Int(file_id,jpim), chain_prof, Gformat )
    
    Call free_chain( chain_prof )

    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(f))
    call rttov_put_lun (file_id)
    
CATCH

  End Subroutine
  
  Subroutine PrintRadiance (err, radiance, name, f, printallrad )
    Use rttov_chain, only : print_array

    Integer(Kind=jpim),  Intent(out) :: err
    Type(radiance_type), Intent(in) :: radiance
    Character*(*),       Intent(in) :: name, f
    Logical(Kind=jplm),  Intent(in) :: printallrad
    
    Integer(Kind=jpim) :: file_id

TRY
    
    call rttov_get_lun (file_id )
    Open( file_id, file = f, form = 'formatted', iostat = err )
    THROWM(err.ne.0,"Cannot open "//Trim(f))

    Write( file_id, '(A," = (")' ) name//'%CLOUDY'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%total )
    Write( file_id, * ) ')'
    
    Write( file_id, '(A," = (")' ) name//'%BT'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%bt )
    Write( file_id, * ) ')'
     
    Write( file_id, '(A," = (")' ) name//'%CLEAR'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%clear )
    Write( file_id, * ) ')'
    
    Write( file_id, '(A," = (")' ) name//'%BT_CLEAR'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%bt_clear )
    Write( file_id, * ) ')'

    Write( file_id, '(A," = (")' ) name//'%OVERCAST'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A2 = radiance%overcast )
    Write( file_id, * ) ')'     

    If (printallrad) Then

      Write( file_id, '(A," = (")' ) name//'%CLOUDY100%'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%cloudy )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%UPCLEAR'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%upclear )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%DNCLEAR'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%dnclear )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%REFLCLEAR'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = radiance%reflclear )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%UP'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A2 = radiance%up )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%DOWN'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A2 = radiance%down )
      Write( file_id, * ) ')'

      Write( file_id, '(A," = (")' ) name//'%SURF'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A2 = radiance%surf )
      Write( file_id, * ) ')'
    
    EndIf

    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(f))
    call rttov_put_lun (file_id)

CATCH
  
  End Subroutine
  
  Subroutine PrintTransmission (err, transmission, name, f )
    Use rttov_chain, only : print_array
!
    Integer(Kind=jpim),      Intent(out) :: err
    Type(transmission_type), Intent(in) :: transmission
    Character*(*),           Intent(in) :: name, f
!

    Integer(Kind=jpim) :: file_id

TRY

    call rttov_get_lun (file_id )
    Open( file_id, file = f, form = 'formatted', iostat = err )
    THROWM(err.ne.0,"Cannot open "//Trim(f))
    
    Write( file_id, '(A," = (")' ) name//'%TAU_TOTAL'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = transmission%tau_total )
    Write( file_id, * ) ')'
    
    Write( file_id, '(A," = (")' ) name//'%TAU_LAYERS'
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A2 = transmission%tau_levels )
    Write( file_id, * ) ')'
     
    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(f))
    call rttov_put_lun (file_id)

CATCH
  
  End Subroutine
  
  
  Subroutine PrintEmissivity (err, emissivity, name, f )
    Use rttov_chain, only : print_array
!
    Integer(Kind=jpim), Intent(out) :: err
    Real(Kind=jprb),    Intent(in)  :: emissivity(:)
    Character*(*),      Intent(in)  :: name, f
!

    Integer(Kind=jpim) :: file_id

TRY


    call rttov_get_lun (file_id )
    Open( file_id, file = f, form = 'formatted', iostat = err )
    THROWM(err.ne.0,"Cannot open "//Trim(f))
    
    Write( file_id, '(A," = (")' ) name
    Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = emissivity )
    Write( file_id, * ) ')'
    
    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(f))
    call rttov_put_lun (file_id)

CATCH
  
  End Subroutine

  Subroutine PrintPCScores(err, pccomp, name, f)
    Use rttov_chain, only : print_array
!
    Integer(Kind=jpim),      Intent(out) :: err
    Type(rttov_pccomp),      Intent(in) :: pccomp
    Character*(*),           Intent(in) :: name, f
!

    Integer(Kind=jpim) :: file_id

TRY

    call rttov_get_lun (file_id )
    Open( file_id, file = f, form = 'formatted', iostat = err )
    THROWM(err.ne.0,"Cannot open "//Trim(f))
    
    If(associated(pccomp%pcscores)) then
      Write( file_id, '(A," = (")' ) name//'%PCSCORES'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = pccomp%pcscores )
      Write( file_id, * ) ')'
    endif
    
    If(associated(pccomp%clear_pccomp)) then
      Write( file_id, '(A," = (")' ) name//'%CLEAR_PCCOMP'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = pccomp%clear_pccomp )
      Write( file_id, * ) ')'
    endif
    
    If(associated(pccomp%bt_pccomp)) then
      Write( file_id, '(A," = (")' ) name//'%BT_PCCOMP'
      Call Print_Array( Int(file_id,jpim), '(10000(' // Trim( Gformat ) // '))', A1 = pccomp%bt_pccomp )
      Write( file_id, * ) ')'
    endif
    
    Close( file_id, iostat = err )
    THROWM(err.ne.0,"Cannot close "//Trim(f))
    call rttov_put_lun (file_id)

CATCH
  

  End Subroutine
  
  Subroutine init_defn (defn)
    Type(rttov_test_defn), Intent(inout) :: defn
    
    defn%f_o3           = ""
    defn%f_co2          = ""
    defn%f_n2o          = ""
    defn%f_co           = ""
    defn%f_ch4          = ""
    defn%f_clw          = ""
    defn%f_aerosl       = ""
    defn%f_cloud        = ""
    defn%f_cfrac        = ""
    defn%f_icede        = ""
    defn%f_pcscores     = ""
    defn%f_channels_rec = ""
  
  End Subroutine


End Program





